# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# This file is generated by wxPython's PI generator.  Do not edit by hand.
#
# The *.pyi files are used by PyCharm and other development tools to provide
# more information, such as PEP 484 type hints, than it is able to glean from
# introspection of extension types and methods.  They are not intended to be
# imported, executed or used for any other purpose other than providing info
# to the tools. If you don't use use a tool that makes use of .pyi files then
# you can safely ignore this file.
#
# See: https://www.python.org/dev/peps/pep-0484/
#      https://www.jetbrains.com/help/pycharm/2016.1/type-hinting-in-pycharm.html
#
# Copyright: (c) 2018 by Total Control Software
# License:   wxWindows License
#---------------------------------------------------------------------------


"""
This module contains a widget class and supporting classes for a generic HTML
renderer.  It supports only a subset of the HTML standards, and no Javascript
or CSS, but it is relatively lightweight and has no platform dependencies.  It
is suitable for displaying simple HTML documents, such as the application's
documentation or built-in help pages.

.. note:: Due to some internal dynamic initialization in wxWidgets, this
          module should be imported **before** the :class:`wx.App` object is
          created.
"""
#-- begin-_html --#

import wx
#-- end-_html --#
#-- begin-htmldefs --#
HTML_ALIGN_LEFT = 0
HTML_ALIGN_RIGHT = 0
HTML_ALIGN_JUSTIFY = 0
HTML_ALIGN_TOP = 0
HTML_ALIGN_BOTTOM = 0
HTML_ALIGN_CENTER = 0
HTML_CLR_FOREGROUND = 0
HTML_CLR_BACKGROUND = 0
HTML_CLR_TRANSPARENT_BACKGROUND = 0
HTML_UNITS_PIXELS = 0
HTML_UNITS_PERCENT = 0
HTML_INDENT_LEFT = 0
HTML_INDENT_RIGHT = 0
HTML_INDENT_TOP = 0
HTML_INDENT_BOTTOM = 0
HTML_INDENT_HORIZONTAL = 0
HTML_INDENT_VERTICAL = 0
HTML_INDENT_ALL = 0
HTML_COND_ISANCHOR = 0
HTML_COND_ISIMAGEMAP = 0
HTML_COND_USER = 0
INT_MAX = 0
#-- end-htmldefs --#
#-- begin-htmlcell --#
HTML_SEL_OUT = 0
HTML_SEL_IN = 0
HTML_SEL_CHANGING = 0
HTML_FIND_EXACT = 0
HTML_FIND_NEAREST_BEFORE = 0
HTML_FIND_NEAREST_AFTER = 0
HTML_SCRIPT_NORMAL = 0
HTML_SCRIPT_SUB = 0
HTML_SCRIPT_SUP = 0

class HtmlSelection(object):
    """
    HtmlSelection()
    """

    def __init__(self):
        """
        HtmlSelection()
        """

    def Set(self, *args, **kw):
        """
        Set(fromPos, fromCell, toPos, toCell)
        Set(fromCell, toCell)
        """

    def GetFromCell(self):
        """
        GetFromCell() -> HtmlCell
        """

    def GetToCell(self):
        """
        GetToCell() -> HtmlCell
        """

    def GetFromPos(self):
        """
        GetFromPos() -> wx.Point
        """

    def GetToPos(self):
        """
        GetToPos() -> wx.Point
        """

    def ClearFromToCharacterPos(self):
        """
        ClearFromToCharacterPos()
        """

    def AreFromToCharacterPosSet(self):
        """
        AreFromToCharacterPosSet() -> bool
        """

    def SetFromCharacterPos(self, pos):
        """
        SetFromCharacterPos(pos)
        """

    def SetToCharacterPos(self, pos):
        """
        SetToCharacterPos(pos)
        """

    def GetFromCharacterPos(self):
        """
        GetFromCharacterPos() -> Coord
        """

    def GetToCharacterPos(self):
        """
        GetToCharacterPos() -> Coord
        """

    def IsEmpty(self):
        """
        IsEmpty() -> bool
        """
    FromCell = property(None, None)
    FromCharacterPos = property(None, None)
    FromPos = property(None, None)
    ToCell = property(None, None)
    ToCharacterPos = property(None, None)
    ToPos = property(None, None)
# end of class HtmlSelection


class HtmlRenderingState(object):
    """
    HtmlRenderingState()
    
    Selection state is passed to wxHtmlCell::Draw so that it can render
    itself differently e.g.
    """

    def __init__(self):
        """
        HtmlRenderingState()
        
        Selection state is passed to wxHtmlCell::Draw so that it can render
        itself differently e.g.
        """

    def SetSelectionState(self, s):
        """
        SetSelectionState(s)
        """

    def GetSelectionState(self):
        """
        GetSelectionState() -> HtmlSelectionState
        """

    def SetFgColour(self, c):
        """
        SetFgColour(c)
        """

    def GetFgColour(self):
        """
        GetFgColour() -> wx.Colour
        """

    def SetBgColour(self, c):
        """
        SetBgColour(c)
        """

    def GetBgColour(self):
        """
        GetBgColour() -> wx.Colour
        """

    def SetBgMode(self, m):
        """
        SetBgMode(m)
        """

    def GetBgMode(self):
        """
        GetBgMode() -> int
        """
    BgColour = property(None, None)
    BgMode = property(None, None)
    FgColour = property(None, None)
    SelectionState = property(None, None)
# end of class HtmlRenderingState


class HtmlRenderingStyle(object):
    """
    wxHtmlSelection is data holder with information about text selection.
    """

    def GetSelectedTextColour(self, clr):
        """
        GetSelectedTextColour(clr) -> wx.Colour
        
        Returns the colour to use for the selected text.
        """

    def GetSelectedTextBgColour(self, clr):
        """
        GetSelectedTextBgColour(clr) -> wx.Colour
        
        Returns the colour to use for the selected text's background.
        """
# end of class HtmlRenderingStyle


class HtmlRenderingInfo(object):
    """
    HtmlRenderingInfo()
    
    This class contains information given to cells when drawing them.
    """

    def __init__(self):
        """
        HtmlRenderingInfo()
        
        This class contains information given to cells when drawing them.
        """

    def SetSelection(self, s):
        """
        SetSelection(s)
        
        Accessors.
        """

    def GetSelection(self):
        """
        GetSelection() -> HtmlSelection
        
        Accessors.
        """

    def SetStyle(self, style):
        """
        SetStyle(style)
        
        Accessors.
        """

    def GetStyle(self):
        """
        GetStyle() -> HtmlRenderingStyle
        
        Accessors.
        """

    def GetState(self):
        """
        GetState() -> HtmlRenderingState
        
        Accessors.
        """
    Selection = property(None, None)
    State = property(None, None)
    Style = property(None, None)
# end of class HtmlRenderingInfo


class HtmlCell(wx.Object):
    """
    HtmlCell()
    
    Internal data structure.
    """

    def __init__(self):
        """
        HtmlCell()
        
        Internal data structure.
        """

    def AdjustPagebreak(self, pagebreak, known_pagebreaks, pageHeight):
        """
        AdjustPagebreak(pagebreak, known_pagebreaks, pageHeight) -> (bool, pagebreak)
        
        This method is used to adjust pagebreak position.
        """

    def Draw(self, dc, x, y, view_y1, view_y2, info):
        """
        Draw(dc, x, y, view_y1, view_y2, info)
        
        Renders the cell.
        """

    def DrawInvisible(self, dc, x, y, info):
        """
        DrawInvisible(dc, x, y, info)
        
        This method is called instead of Draw() when the cell is certainly out
        of the screen (and thus invisible).
        """

    def Find(self, condition, param):
        """
        Find(condition, param) -> HtmlCell
        
        Returns pointer to itself if this cell matches condition (or if any of
        the cells following in the list matches), NULL otherwise.
        """

    def FindCellByPos(self, x, y, flags=HTML_FIND_EXACT):
        """
        FindCellByPos(x, y, flags=HTML_FIND_EXACT) -> HtmlCell
        
        Find a cell inside this cell positioned at the given coordinates
        (relative to this's positions).
        """

    def GetDescent(self):
        """
        GetDescent() -> int
        
        Returns descent value of the cell (m_Descent member).
        """

    def GetFirstChild(self):
        """
        GetFirstChild() -> HtmlCell
        
        Returns pointer to the first cell in the list.
        """

    def GetHeight(self):
        """
        GetHeight() -> int
        
        Returns height of the cell (m_Height member).
        """

    def GetId(self):
        """
        GetId() -> String
        
        Returns unique cell identifier if there is any, the empty string
        otherwise.
        """

    def GetLink(self, x=0, y=0):
        """
        GetLink(x=0, y=0) -> HtmlLinkInfo
        
        Returns hypertext link if associated with this cell or NULL otherwise.
        """

    def GetMouseCursor(self, window):
        """
        GetMouseCursor(window) -> wx.Cursor
        
        Returns cursor to show when mouse pointer is over the cell.
        """

    def GetMouseCursorAt(self, window, rePos):
        """
        GetMouseCursorAt(window, rePos) -> wx.Cursor
        
        Returns cursor to show when mouse pointer is over the specified point.
        """

    def GetNext(self):
        """
        GetNext() -> HtmlCell
        
        Returns pointer to the next cell in list (see htmlcell.h if you're
        interested in details).
        """

    def GetParent(self):
        """
        GetParent() -> HtmlContainerCell
        
        Returns pointer to parent container.
        """

    def GetPosX(self):
        """
        GetPosX() -> int
        
        Returns X position within parent (the value is relative to parent's
        upper left corner).
        """

    def GetPosY(self):
        """
        GetPosY() -> int
        
        Returns Y position within parent (the value is relative to parent's
        upper left corner).
        """

    def GetWidth(self):
        """
        GetWidth() -> int
        
        Returns width of the cell (m_Width member).
        """

    def Layout(self, w):
        """
        Layout(w)
        
        Layouts the cell.
        """

    def ProcessMouseClick(self, window, pos, event):
        """
        ProcessMouseClick(window, pos, event) -> bool
        
        This function is simple event handler.
        """

    def SetId(self, id):
        """
        SetId(id)
        
        Sets unique cell identifier.
        """

    def SetLink(self, link):
        """
        SetLink(link)
        
        Sets the hypertext link associated with this cell.
        """

    def SetNext(self, cell):
        """
        SetNext(cell)
        
        Sets the next cell in the list.
        """

    def SetParent(self, p):
        """
        SetParent(p)
        
        Sets parent container of this cell.
        """

    def SetPos(self, x, y):
        """
        SetPos(x, y)
        
        Sets the cell's position within parent container.
        """

    def ConvertToText(self, sel):
        """
        ConvertToText(sel) -> String
        
        Converts the cell into text representation.
        """
    Descent = property(None, None)
    FirstChild = property(None, None)
    Height = property(None, None)
    Id = property(None, None)
    Link = property(None, None)
    Next = property(None, None)
    Parent = property(None, None)
    PosX = property(None, None)
    PosY = property(None, None)
    Width = property(None, None)
# end of class HtmlCell


class HtmlContainerCell(HtmlCell):
    """
    HtmlContainerCell(parent)
    
    The wxHtmlContainerCell class is an implementation of a cell that may
    contain more cells in it.
    """

    def __init__(self, parent):
        """
        HtmlContainerCell(parent)
        
        The wxHtmlContainerCell class is an implementation of a cell that may
        contain more cells in it.
        """

    def GetAlignHor(self):
        """
        GetAlignHor() -> int
        
        Returns container's horizontal alignment.
        """

    def GetAlignVer(self):
        """
        GetAlignVer() -> int
        
        Returns container's vertical alignment.
        """

    def GetBackgroundColour(self):
        """
        GetBackgroundColour() -> wx.Colour
        
        Returns the background colour of the container or wxNullColour if no
        background colour is set.
        """

    def GetIndent(self, ind):
        """
        GetIndent(ind) -> int
        
        Returns the indentation.
        """

    def GetIndentUnits(self, ind):
        """
        GetIndentUnits(ind) -> int
        
        Returns the units of indentation for ind where ind is one of the
        wxHTML_INDENT_* constants.
        """

    def InsertCell(self, cell):
        """
        InsertCell(cell)
        
        Inserts a new cell into the container.
        """

    def SetAlign(self, tag):
        """
        SetAlign(tag)
        
        Sets the container's alignment (both horizontal and vertical)
        according to the values stored in tag.
        """

    def SetAlignHor(self, al):
        """
        SetAlignHor(al)
        
        Sets the container's horizontal alignment.
        """

    def SetAlignVer(self, al):
        """
        SetAlignVer(al)
        
        Sets the container's vertical alignment.
        """

    def SetBackgroundColour(self, clr):
        """
        SetBackgroundColour(clr)
        
        Sets the background colour for this container.
        """

    def SetBorder(self, clr1, clr2, border=1):
        """
        SetBorder(clr1, clr2, border=1)
        
        Sets the border (frame) colours.
        """

    def SetIndent(self, i, what, units=HTML_UNITS_PIXELS):
        """
        SetIndent(i, what, units=HTML_UNITS_PIXELS)
        
        Sets the indentation (free space between borders of container and
        subcells).
        """

    def SetMinHeight(self, h, align=HTML_ALIGN_TOP):
        """
        SetMinHeight(h, align=HTML_ALIGN_TOP)
        
        Sets minimal height of the container.
        """

    def SetWidthFloat(self, *args, **kw):
        """
        SetWidthFloat(w, units)
        SetWidthFloat(tag, pixel_scale=1.0)
        
        Sets floating width adjustment.
        """
    AlignHor = property(None, None)
    AlignVer = property(None, None)
    BackgroundColour = property(None, None)
# end of class HtmlContainerCell


class HtmlLinkInfo(wx.Object):
    """
    HtmlLinkInfo()
    HtmlLinkInfo(href, target=wx.EmptyString)
    
    This class stores all necessary information about hypertext links (as
    represented by <A> tag in HTML documents).
    """

    def __init__(self, *args, **kw):
        """
        HtmlLinkInfo()
        HtmlLinkInfo(href, target=wx.EmptyString)
        
        This class stores all necessary information about hypertext links (as
        represented by <A> tag in HTML documents).
        """

    def GetEvent(self):
        """
        GetEvent() -> wx.MouseEvent
        
        Return pointer to event that generated OnLinkClicked() event.
        """

    def GetHref(self):
        """
        GetHref() -> String
        
        Return HREF value of the <A> tag.
        """

    def GetHtmlCell(self):
        """
        GetHtmlCell() -> HtmlCell
        
        Return pointer to the cell that was clicked.
        """

    def GetTarget(self):
        """
        GetTarget() -> String
        
        Return TARGET value of the <A> tag (this value is used to specify in
        which frame should be the page pointed by GetHref() Href opened).
        """
    Event = property(None, None)
    Href = property(None, None)
    HtmlCell = property(None, None)
    Target = property(None, None)
# end of class HtmlLinkInfo


class HtmlColourCell(HtmlCell):
    """
    HtmlColourCell(clr, flags=HTML_CLR_FOREGROUND)
    
    This cell changes the colour of either the background or the
    foreground.
    """

    def __init__(self, clr, flags=HTML_CLR_FOREGROUND):
        """
        HtmlColourCell(clr, flags=HTML_CLR_FOREGROUND)
        
        This cell changes the colour of either the background or the
        foreground.
        """
# end of class HtmlColourCell


class HtmlWidgetCell(HtmlCell):
    """
    HtmlWidgetCell(wnd, w=0)
    
    wxHtmlWidgetCell is a class that provides a connection between HTML
    cells and widgets (an object derived from wxWindow).
    """

    def __init__(self, wnd, w=0):
        """
        HtmlWidgetCell(wnd, w=0)
        
        wxHtmlWidgetCell is a class that provides a connection between HTML
        cells and widgets (an object derived from wxWindow).
        """
# end of class HtmlWidgetCell


class HtmlWordCell(HtmlCell):
    """
    HtmlWordCell(word, dc)
    
    This html cell represents a single word or text fragment in the
    document stream.
    """

    def __init__(self, word, dc):
        """
        HtmlWordCell(word, dc)
        
        This html cell represents a single word or text fragment in the
        document stream.
        """
# end of class HtmlWordCell


class HtmlWordWithTabsCell(HtmlWordCell):
    """
    HtmlWordWithTabsCell(word, wordOrig, linepos, dc)
    
    wxHtmlWordCell is a specialization for storing text fragments with
    embedded tab characters.
    """

    def __init__(self, word, wordOrig, linepos, dc):
        """
        HtmlWordWithTabsCell(word, wordOrig, linepos, dc)
        
        wxHtmlWordCell is a specialization for storing text fragments with
        embedded tab characters.
        """
# end of class HtmlWordWithTabsCell


class HtmlFontCell(HtmlCell):
    """
    HtmlFontCell(font)
    
    This cell represents a font change in the document stream.
    """

    def __init__(self, font):
        """
        HtmlFontCell(font)
        
        This cell represents a font change in the document stream.
        """
# end of class HtmlFontCell

#-- end-htmlcell --#
#-- begin-htmlfilt --#

class HtmlFilter(wx.Object):
    """
    HtmlFilter()
    
    This class is the parent class of input filters for wxHtmlWindow.
    """

    def __init__(self):
        """
        HtmlFilter()
        
        This class is the parent class of input filters for wxHtmlWindow.
        """

    def CanRead(self, file):
        """
        CanRead(file) -> bool
        
        Returns true if this filter is capable of reading file file.
        """

    def ReadFile(self, file):
        """
        ReadFile(file) -> String
        
        Reads the file and returns string with HTML document.
        """
# end of class HtmlFilter

#-- end-htmlfilt --#
#-- begin-htmltag --#

class HtmlTag(object):
    """
    This class represents a single HTML tag.
    """

    def GetAllParams(self):
        """
        GetAllParams() -> String
        
        Returns a string containing all parameters.
        """

    def GetBeginPos(self):
        """
        GetBeginPos() -> int
        
        Returns beginning position of the text between this tag and paired
        ending tag.
        """

    def GetEndPos1(self):
        """
        GetEndPos1() -> int
        
        Returns ending position of the text between this tag and paired ending
        tag.
        """

    def GetEndPos2(self):
        """
        GetEndPos2() -> int
        
        Returns ending position 2 of the text between this tag and paired
        ending tag.
        """

    def GetName(self):
        """
        GetName() -> String
        
        Returns tag's name.
        """

    def GetParam(self, par, with_quotes=False):
        """
        GetParam(par, with_quotes=False) -> String
        
        Returns the value of the parameter.
        """

    def GetParamAsColour(self, par):
        """
        GetParamAsColour(par) -> (bool, clr)
        
        Interprets tag parameter par as colour specification and saves its
        value into wxColour variable pointed by clr.
        """

    def GetParamAsInt(self, par):
        """
        GetParamAsInt(par) -> (bool, value)
        
        Interprets tag parameter par as an integer and saves its value into
        int variable pointed by value.
        """

    def GetParamAsString(self, par, value):
        """
        GetParamAsString(par, value) -> bool
        
        Get the value of the parameter.
        """

    def HasEnding(self):
        """
        HasEnding() -> bool
        
        Returns true if this tag is paired with ending tag, false otherwise.
        """

    def HasParam(self, par):
        """
        HasParam(par) -> bool
        
        Returns true if the tag has a parameter of the given name.
        """

    @staticmethod
    def ParseAsColour(str):
        """
        ParseAsColour(str) -> (bool, clr)
        
        Parses the given string as an HTML colour.
        """
    AllParams = property(None, None)
    BeginPos = property(None, None)
    EndPos1 = property(None, None)
    EndPos2 = property(None, None)
    Name = property(None, None)
# end of class HtmlTag

#-- end-htmltag --#
#-- begin-htmlpars --#
HTML_URL_PAGE = 0
HTML_URL_IMAGE = 0
HTML_URL_OTHER = 0

class HtmlTagHandler(wx.Object):
    """
    HtmlTagHandler()
    """

    def __init__(self):
        """
        HtmlTagHandler()
        """

    def GetSupportedTags(self):
        """
        GetSupportedTags() -> String
        
        Returns list of supported tags.
        """

    def HandleTag(self, tag):
        """
        HandleTag(tag) -> bool
        
        This is the core method of each handler.
        """

    def SetParser(self, parser):
        """
        SetParser(parser)
        
        Assigns parser to this handler.
        """

    def GetParser(self):
        """
        GetParser() -> HtmlParser
        
        Returns the parser associated with this tag handler.
        """
    Parser = property(None, None)
    SupportedTags = property(None, None)

    def ParseInner(self, tag):
        """
        ParseInner(tag)
        
        This method calls parser's wxHtmlParser::DoParsing method for the
        string between this tag and the paired ending tag:
        """

    def ParseInnerSource(self, source):
        """
        ParseInnerSource(source)
        
        Parses given source as if it was tag's inner code (see
        wxHtmlParser::GetInnerSource).
        """
# end of class HtmlTagHandler


class HtmlParser(object):
    """
    HtmlParser()
    
    Classes derived from this handle the generic parsing of HTML
    documents: it scans the document and divide it into blocks of tags
    (where one block consists of beginning and ending tag and of text
    between these two tags).
    """

    def __init__(self):
        """
        HtmlParser()
        
        Classes derived from this handle the generic parsing of HTML
        documents: it scans the document and divide it into blocks of tags
        (where one block consists of beginning and ending tag and of text
        between these two tags).
        """

    def AddTagHandler(self, handler):
        """
        AddTagHandler(handler)
        
        Adds handler to the internal list ( hash table) of handlers.
        """

    def DoParsing(self):
        """
        DoParsing()
        
        Parses the whole m_Source.
        """

    def DoneParser(self):
        """
        DoneParser()
        
        This must be called after DoParsing().
        """

    def GetFS(self):
        """
        GetFS() -> wx.FileSystem
        
        Returns pointer to the file system.
        """

    def GetProduct(self):
        """
        GetProduct() -> wx.Object
        
        Returns product of parsing.
        """

    def GetSource(self):
        """
        GetSource() -> String
        
        Returns pointer to the source being parsed.
        """

    def InitParser(self, source):
        """
        InitParser(source)
        
        Setups the parser for parsing the source string.
        """

    def OpenURL(self, type, url):
        """
        OpenURL(type, url) -> wx.FSFile
        
        Opens given URL and returns wxFSFile object that can be used to read
        data from it.
        """

    def Parse(self, source):
        """
        Parse(source) -> wx.Object
        
        Proceeds parsing of the document.
        """

    def PopTagHandler(self):
        """
        PopTagHandler()
        
        Restores parser's state before last call to PushTagHandler().
        """

    def PushTagHandler(self, handler, tags):
        """
        PushTagHandler(handler, tags)
        
        Forces the handler to handle additional tags (not returned by
        wxHtmlTagHandler::GetSupportedTags).
        """

    def SetFS(self, fs):
        """
        SetFS(fs)
        
        Sets the virtual file system that will be used to request additional
        files.
        """

    def StopParsing(self):
        """
        StopParsing()
        
        Call this function to interrupt parsing from a tag handler.
        """
    FS = property(None, None)
    Product = property(None, None)
    Source = property(None, None)

    def AddTag(self, tag):
        """
        AddTag(tag)
        
        This may (and may not) be overwritten in derived class.
        """
# end of class HtmlParser

#-- end-htmlpars --#
#-- begin-htmlwin --#
HW_SCROLLBAR_NEVER = 0
HW_SCROLLBAR_AUTO = 0
HW_NO_SELECTION = 0
HW_DEFAULT_STYLE = 0
HTML_OPEN = 0
HTML_BLOCK = 0
HTML_REDIRECT = 0
wxEVT_HTML_CELL_CLICKED = 0
wxEVT_HTML_CELL_HOVER = 0
wxEVT_HTML_LINK_CLICKED = 0

class HtmlWindowInterface(object):
    """
    HtmlWindowInterface()
    
    Abstract interface to a HTML rendering window (such as wxHtmlWindow or
    wxHtmlListBox) that is passed to wxHtmlWinParser.
    """
    HTMLCursor_Default = 0
    HTMLCursor_Link = 0
    HTMLCursor_Text = 0

    def __init__(self):
        """
        HtmlWindowInterface()
        
        Abstract interface to a HTML rendering window (such as wxHtmlWindow or
        wxHtmlListBox) that is passed to wxHtmlWinParser.
        """

    def SetHTMLWindowTitle(self, title):
        """
        SetHTMLWindowTitle(title)
        
        Called by the parser to set window's title to given text.
        """

    def OnHTMLLinkClicked(self, link):
        """
        OnHTMLLinkClicked(link)
        
        Called when a link is clicked.
        """

    def OnHTMLOpeningURL(self, type, url, redirect):
        """
        OnHTMLOpeningURL(type, url, redirect) -> HtmlOpeningStatus
        
        Called when the parser needs to open another URL (e.g.
        """

    def HTMLCoordsToWindow(self, cell, pos):
        """
        HTMLCoordsToWindow(cell, pos) -> wx.Point
        
        Converts coordinates pos relative to given cell to physical
        coordinates in the window.
        """

    def GetHTMLWindow(self):
        """
        GetHTMLWindow() -> wx.Window
        
        Returns the window used for rendering (may be NULL).
        """

    def GetHTMLBackgroundColour(self):
        """
        GetHTMLBackgroundColour() -> wx.Colour
        
        Returns background colour to use by default.
        """

    def SetHTMLBackgroundColour(self, clr):
        """
        SetHTMLBackgroundColour(clr)
        
        Sets window's background to colour clr.
        """

    def SetHTMLBackgroundImage(self, bmpBg):
        """
        SetHTMLBackgroundImage(bmpBg)
        
        Sets window's background to given bitmap.
        """

    def SetHTMLStatusText(self, text):
        """
        SetHTMLStatusText(text)
        
        Sets status bar text.
        """

    def GetHTMLCursor(self, type):
        """
        GetHTMLCursor(type) -> wx.Cursor
        
        Returns mouse cursor of given type.
        """
    HTMLBackgroundColour = property(None, None)
    HTMLWindow = property(None, None)
# end of class HtmlWindowInterface


class HtmlWindow(wx.ScrolledWindow, HtmlWindowInterface):
    """
    HtmlWindow()
    HtmlWindow(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=HW_DEFAULT_STYLE, name="htmlWindow")
    
    wxHtmlWindow is probably the only class you will directly use unless
    you want to do something special (like adding new tag handlers or MIME
    filters).
    """

    def __init__(self, *args, **kw):
        """
        HtmlWindow()
        HtmlWindow(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=HW_DEFAULT_STYLE, name="htmlWindow")
        
        wxHtmlWindow is probably the only class you will directly use unless
        you want to do something special (like adding new tag handlers or MIME
        filters).
        """

    def AppendToPage(self, source):
        """
        AppendToPage(source) -> bool
        
        Appends HTML fragment to currently displayed text and refreshes the
        window.
        """

    def GetInternalRepresentation(self):
        """
        GetInternalRepresentation() -> HtmlContainerCell
        
        Returns pointer to the top-level container.
        """

    def GetOpenedAnchor(self):
        """
        GetOpenedAnchor() -> String
        
        Returns anchor within currently opened page (see
        wxHtmlWindow::GetOpenedPage).
        """

    def GetOpenedPage(self):
        """
        GetOpenedPage() -> String
        
        Returns full location of the opened page.
        """

    def GetOpenedPageTitle(self):
        """
        GetOpenedPageTitle() -> String
        
        Returns title of the opened page or wxEmptyString if the current page
        does not contain <TITLE> tag.
        """

    def GetParser(self):
        """
        GetParser() -> HtmlWinParser
        
        Returns a pointer to the current parser.
        """

    def GetRelatedFrame(self):
        """
        GetRelatedFrame() -> wx.Frame
        
        Returns the related frame.
        """

    def HistoryBack(self):
        """
        HistoryBack() -> bool
        
        Moves back to the previous page.
        """

    def HistoryCanBack(self):
        """
        HistoryCanBack() -> bool
        
        Returns true if it is possible to go back in the history i.e.
        """

    def HistoryCanForward(self):
        """
        HistoryCanForward() -> bool
        
        Returns true if it is possible to go forward in the history i.e.
        """

    def HistoryClear(self):
        """
        HistoryClear()
        
        Clears history.
        """

    def HistoryForward(self):
        """
        HistoryForward() -> bool
        
        Moves to next page in history.
        """

    def LoadFile(self, filename):
        """
        LoadFile(filename) -> bool
        
        Loads an HTML page from a file and displays it.
        """

    def LoadPage(self, location):
        """
        LoadPage(location) -> bool
        
        Unlike SetPage() this function first loads the HTML page from location
        and then displays it.
        """

    def OnLinkClicked(self, link):
        """
        OnLinkClicked(link)
        
        Called when user clicks on hypertext link.
        """

    def OnOpeningURL(self, type, url, redirect):
        """
        OnOpeningURL(type, url, redirect) -> HtmlOpeningStatus
        
        Called when an URL is being opened (either when the user clicks on a
        link or an image is loaded).
        """

    def OnSetTitle(self, title):
        """
        OnSetTitle(title)
        
        Called on parsing <TITLE> tag.
        """

    def ReadCustomization(self, cfg, path=wx.EmptyString):
        """
        ReadCustomization(cfg, path=wx.EmptyString)
        
        This reads custom settings from wxConfig.
        """

    def SelectAll(self):
        """
        SelectAll()
        
        Selects all text in the window.
        """

    def SelectLine(self, pos):
        """
        SelectLine(pos)
        
        Selects the line of text that pos points at.
        """

    def SelectWord(self, pos):
        """
        SelectWord(pos)
        
        Selects the word at position pos.
        """

    def SelectionToText(self):
        """
        SelectionToText() -> String
        
        Returns the current selection as plain text.
        """

    def SetBorders(self, b):
        """
        SetBorders(b)
        
        This function sets the space between border of window and HTML
        contents.
        """

    def SetFonts(self, normal_face, fixed_face, sizes):
        """
        SetFonts(normal_face, fixed_face, sizes)
        
        This function sets font sizes and faces.
        """

    def SetStandardFonts(self, size=-1, normal_face=wx.EmptyString, fixed_face=wx.EmptyString):
        """
        SetStandardFonts(size=-1, normal_face=wx.EmptyString, fixed_face=wx.EmptyString)
        
        Sets default font sizes and/or default font size.
        """

    def SetPage(self, source):
        """
        SetPage(source) -> bool
        
        Sets the source of a page and displays it, for example:
        """

    def SetRelatedFrame(self, frame, format):
        """
        SetRelatedFrame(frame, format)
        
        Sets the frame in which page title will be displayed.
        """

    def SetRelatedStatusBar(self, *args, **kw):
        """
        SetRelatedStatusBar(index)
        SetRelatedStatusBar(statusbar, index=0)
        
        After calling SetRelatedFrame(), this sets statusbar slot where
        messages will be displayed.
        """

    def ToText(self):
        """
        ToText() -> String
        
        Returns content of currently displayed page as plain text.
        """

    def WriteCustomization(self, cfg, path=wx.EmptyString):
        """
        WriteCustomization(cfg, path=wx.EmptyString)
        
        Saves custom settings into wxConfig.
        """

    @staticmethod
    def AddFilter(filter):
        """
        AddFilter(filter)
        
        Adds input filter to the static list of available filters.
        """
    InternalRepresentation = property(None, None)
    OpenedAnchor = property(None, None)
    OpenedPage = property(None, None)
    OpenedPageTitle = property(None, None)
    Parser = property(None, None)
    RelatedFrame = property(None, None)

    def OnCellClicked(self, cell, x, y, event):
        """
        OnCellClicked(cell, x, y, event) -> bool
        
        This method is called when a mouse button is clicked inside
        wxHtmlWindow.
        """

    def OnCellMouseHover(self, cell, x, y):
        """
        OnCellMouseHover(cell, x, y)
        
        This method is called when a mouse moves over an HTML cell.
        """
# end of class HtmlWindow


class HtmlLinkEvent(wx.CommandEvent):
    """
    HtmlLinkEvent(id, linkinfo)
    
    This event class is used for the events generated by wxHtmlWindow.
    """

    def __init__(self, id, linkinfo):
        """
        HtmlLinkEvent(id, linkinfo)
        
        This event class is used for the events generated by wxHtmlWindow.
        """

    def GetLinkInfo(self):
        """
        GetLinkInfo() -> HtmlLinkInfo
        
        Returns the wxHtmlLinkInfo which contains info about the cell clicked
        and the hyperlink it contains.
        """
    LinkInfo = property(None, None)
# end of class HtmlLinkEvent


class HtmlCellEvent(wx.CommandEvent):
    """
    HtmlCellEvent(commandType, id, cell, point, ev)
    
    This event class is used for the events generated by wxHtmlWindow.
    """

    def __init__(self, commandType, id, cell, point, ev):
        """
        HtmlCellEvent(commandType, id, cell, point, ev)
        
        This event class is used for the events generated by wxHtmlWindow.
        """

    def GetCell(self):
        """
        GetCell() -> HtmlCell
        
        Returns the wxHtmlCellEvent associated with the event.
        """

    def GetLinkClicked(self):
        """
        GetLinkClicked() -> bool
        
        Returns true if SetLinkClicked(true) has previously been called; false
        otherwise.
        """

    def GetPoint(self):
        """
        GetPoint() -> wx.Point
        
        Returns the wxPoint associated with the event.
        """

    def SetLinkClicked(self, linkclicked):
        """
        SetLinkClicked(linkclicked)
        
        Call this function with linkclicked set to true if the cell which has
        been clicked contained a link or false otherwise (which is the
        default).
        """
    Cell = property(None, None)
    LinkClicked = property(None, None)
    Point = property(None, None)
# end of class HtmlCellEvent


EVT_HTML_CELL_CLICKED = wx.PyEventBinder( wxEVT_HTML_CELL_CLICKED, 1 )
EVT_HTML_CELL_HOVER   = wx.PyEventBinder( wxEVT_HTML_CELL_HOVER, 1 )
EVT_HTML_LINK_CLICKED = wx.PyEventBinder( wxEVT_HTML_LINK_CLICKED, 1 )

# deprecated wxEVT aliases
wxEVT_COMMAND_HTML_CELL_CLICKED  = wxEVT_HTML_CELL_CLICKED
wxEVT_COMMAND_HTML_CELL_HOVER    = wxEVT_HTML_CELL_HOVER
wxEVT_COMMAND_HTML_LINK_CLICKED  = wxEVT_HTML_LINK_CLICKED
#-- end-htmlwin --#
#-- begin-htmlprint --#
PAGE_ODD = 0
PAGE_EVEN = 0
PAGE_ALL = 0

class HtmlDCRenderer(wx.Object):
    """
    HtmlDCRenderer()
    
    This class can render HTML document into a specified area of a DC.
    """

    def __init__(self):
        """
        HtmlDCRenderer()
        
        This class can render HTML document into a specified area of a DC.
        """

    def GetTotalWidth(self):
        """
        GetTotalWidth() -> int
        
        Returns the width of the HTML text in pixels.
        """

    def GetTotalHeight(self):
        """
        GetTotalHeight() -> int
        
        Returns the height of the HTML text in pixels.
        """

    def Render(self, x, y, known_pagebreaks, from_=0, dont_render=False, to_=INT_MAX):
        """
        Render(x, y, known_pagebreaks, from_=0, dont_render=False, to_=INT_MAX) -> int
        
        Renders HTML text to the DC.
        """

    def SetDC(self, dc, pixel_scale=1.0):
        """
        SetDC(dc, pixel_scale=1.0)
        
        Assign DC instance to the renderer.
        """

    def SetFonts(self, normal_face, fixed_face, sizes):
        """
        SetFonts(normal_face, fixed_face, sizes)
        
        This function sets font sizes and faces.
        """

    def SetStandardFonts(self, size=-1, normal_face=wx.EmptyString, fixed_face=wx.EmptyString):
        """
        SetStandardFonts(size=-1, normal_face=wx.EmptyString, fixed_face=wx.EmptyString)
        
        Sets font sizes to be relative to the given size or the system default
        size; use either specified or default font.
        """

    def SetHtmlText(self, html, basepath=wx.EmptyString, isdir=True):
        """
        SetHtmlText(html, basepath=wx.EmptyString, isdir=True)
        
        Assign text to the renderer.
        """

    def SetSize(self, width, height):
        """
        SetSize(width, height)
        
        Set size of output rectangle, in pixels.
        """
    TotalHeight = property(None, None)
    TotalWidth = property(None, None)
# end of class HtmlDCRenderer


class HtmlEasyPrinting(wx.Object):
    """
    HtmlEasyPrinting(name="Printing", parentWindow=None)
    
    This class provides very simple interface to printing architecture.
    """

    def __init__(self, name="Printing", parentWindow=None):
        """
        HtmlEasyPrinting(name="Printing", parentWindow=None)
        
        This class provides very simple interface to printing architecture.
        """

    def GetName(self):
        """
        GetName() -> String
        
        Returns the current name being used for preview frames and setup
        dialogs.
        """

    def GetPageSetupData(self):
        """
        GetPageSetupData() -> wx.PageSetupDialogData
        
        Returns a pointer to wxPageSetupDialogData instance used by this
        class.
        """

    def GetParentWindow(self):
        """
        GetParentWindow() -> wx.Window
        
        Gets the parent window for dialogs.
        """

    def GetPrintData(self):
        """
        GetPrintData() -> wx.PrintData
        
        Returns pointer to wxPrintData instance used by this class.
        """

    def PageSetup(self):
        """
        PageSetup()
        
        Display page setup dialog and allows the user to modify settings.
        """

    def PreviewFile(self, htmlfile):
        """
        PreviewFile(htmlfile) -> bool
        
        Preview HTML file.
        """

    def PreviewText(self, htmltext, basepath=wx.EmptyString):
        """
        PreviewText(htmltext, basepath=wx.EmptyString) -> bool
        
        Preview HTML text (not file!).
        """

    def PrintFile(self, htmlfile):
        """
        PrintFile(htmlfile) -> bool
        
        Print HTML file.
        """

    def PrintText(self, htmltext, basepath=wx.EmptyString):
        """
        PrintText(htmltext, basepath=wx.EmptyString) -> bool
        
        Print HTML text (not file!).
        """

    def SetFonts(self, normal_face, fixed_face, sizes):
        """
        SetFonts(normal_face, fixed_face, sizes)
        
        Sets fonts.
        """

    def SetName(self, name):
        """
        SetName(name)
        
        Sets the name used for preview frames and setup dialogs.
        """

    def SetStandardFonts(self, size=-1, normal_face=wx.EmptyString, fixed_face=wx.EmptyString):
        """
        SetStandardFonts(size=-1, normal_face=wx.EmptyString, fixed_face=wx.EmptyString)
        
        Sets default font sizes and/or default font size.
        """

    def SetFooter(self, footer, pg=PAGE_ALL):
        """
        SetFooter(footer, pg=PAGE_ALL)
        
        Set page footer.
        """

    def SetHeader(self, header, pg=PAGE_ALL):
        """
        SetHeader(header, pg=PAGE_ALL)
        
        Set page header.
        """

    def SetParentWindow(self, window):
        """
        SetParentWindow(window)
        
        Sets the parent window for dialogs.
        """
    Name = property(None, None)
    PageSetupData = property(None, None)
    ParentWindow = property(None, None)
    PrintData = property(None, None)
# end of class HtmlEasyPrinting


class HtmlPrintout(wx.Printout):
    """
    HtmlPrintout(title="Printout")
    
    This class serves as printout class for HTML documents.
    """

    def __init__(self, title="Printout"):
        """
        HtmlPrintout(title="Printout")
        
        This class serves as printout class for HTML documents.
        """

    def SetFonts(self, normal_face, fixed_face, sizes):
        """
        SetFonts(normal_face, fixed_face, sizes)
        
        This function sets font sizes and faces.
        """

    def SetFooter(self, footer, pg=PAGE_ALL):
        """
        SetFooter(footer, pg=PAGE_ALL)
        
        Set page footer.
        """

    def SetHeader(self, header, pg=PAGE_ALL):
        """
        SetHeader(header, pg=PAGE_ALL)
        
        Set page header.
        """

    def SetHtmlFile(self, htmlfile):
        """
        SetHtmlFile(htmlfile)
        
        Prepare the class for printing this HTML file.
        """

    def SetHtmlText(self, html, basepath=wx.EmptyString, isdir=True):
        """
        SetHtmlText(html, basepath=wx.EmptyString, isdir=True)
        
        Prepare the class for printing this HTML text.
        """

    def SetMargins(self, top=25.2, bottom=25.2, left=25.2, right=25.2, spaces=5):
        """
        SetMargins(top=25.2, bottom=25.2, left=25.2, right=25.2, spaces=5)
        
        Sets margins in millimeters.
        """

    @staticmethod
    def AddFilter(filter):
        """
        AddFilter(filter)
        
        Adds a filter to the static list of filters for wxHtmlPrintout.
        """
# end of class HtmlPrintout

#-- end-htmlprint --#
#-- begin-htmlwinpars --#

class HtmlWinTagHandler(HtmlTagHandler):
    """
    HtmlWinTagHandler()
    
    This is basically wxHtmlTagHandler except that it is extended with
    protected member m_WParser pointing to the wxHtmlWinParser object
    (value of this member is identical to wxHtmlParser's m_Parser).
    """

    def __init__(self):
        """
        HtmlWinTagHandler()
        
        This is basically wxHtmlTagHandler except that it is extended with
        protected member m_WParser pointing to the wxHtmlWinParser object
        (value of this member is identical to wxHtmlParser's m_Parser).
        """

    def SetParser(self, parser):
        """
        SetParser(parser)
        
        Assigns parser to this handler.
        """

    def GetParser(self):
        """
        GetParser() -> HtmlWinParser
        """
    Parser = property(None, None)
# end of class HtmlWinTagHandler


class HtmlWinParser(HtmlParser):
    """
    HtmlWinParser(wndIface=None)
    
    This class is derived from wxHtmlParser and its main goal is to parse
    HTML input so that it can be displayed in wxHtmlWindow.
    """

    def __init__(self, wndIface=None):
        """
        HtmlWinParser(wndIface=None)
        
        This class is derived from wxHtmlParser and its main goal is to parse
        HTML input so that it can be displayed in wxHtmlWindow.
        """

    def CloseContainer(self):
        """
        CloseContainer() -> HtmlContainerCell
        
        Closes the container, sets actual container to the parent one and
        returns pointer to it (see Cells and Containers).
        """

    def CreateCurrentFont(self):
        """
        CreateCurrentFont() -> wx.Font
        
        Creates font based on current setting (see SetFontSize(),
        SetFontBold(), SetFontItalic(), SetFontFixed(),
        wxHtmlWinParser::SetFontUnderlined) and returns pointer to it.
        """

    def GetActualColor(self):
        """
        GetActualColor() -> wx.Colour
        
        Returns actual text colour.
        """

    def GetAlign(self):
        """
        GetAlign() -> int
        
        Returns default horizontal alignment.
        """

    def GetCharHeight(self):
        """
        GetCharHeight() -> int
        
        Returns (average) char height in standard font.
        """

    def GetCharWidth(self):
        """
        GetCharWidth() -> int
        
        Returns average char width in standard font.
        """

    def GetContainer(self):
        """
        GetContainer() -> HtmlContainerCell
        
        Returns pointer to the currently opened container (see Cells and
        Containers).
        """

    def GetDC(self):
        """
        GetDC() -> wx.DC
        
        Returns pointer to the DC used during parsing.
        """

    def GetFontBold(self):
        """
        GetFontBold() -> int
        
        Returns true if actual font is bold, false otherwise.
        """

    def GetFontFace(self):
        """
        GetFontFace() -> String
        
        Returns actual font face name.
        """

    def GetFontFixed(self):
        """
        GetFontFixed() -> int
        
        Returns true if actual font is fixed face, false otherwise.
        """

    def GetFontItalic(self):
        """
        GetFontItalic() -> int
        
        Returns true if actual font is italic, false otherwise.
        """

    def GetFontSize(self):
        """
        GetFontSize() -> int
        
        Returns actual font size (HTML size varies from -2 to +4)
        """

    def GetFontUnderlined(self):
        """
        GetFontUnderlined() -> int
        
        Returns true if actual font is underlined, false otherwise.
        """

    def GetLink(self):
        """
        GetLink() -> HtmlLinkInfo
        
        Returns actual hypertext link.
        """

    def GetLinkColor(self):
        """
        GetLinkColor() -> wx.Colour
        
        Returns the colour of hypertext link text.
        """

    def GetWindowInterface(self):
        """
        GetWindowInterface() -> HtmlWindowInterface
        
        Returns associated window (wxHtmlWindow).
        """

    def OpenContainer(self):
        """
        OpenContainer() -> HtmlContainerCell
        
        Opens new container and returns pointer to it (see Cells and
        Containers).
        """

    def SetActualColor(self, clr):
        """
        SetActualColor(clr)
        
        Sets actual text colour.
        """

    def SetAlign(self, a):
        """
        SetAlign(a)
        
        Sets default horizontal alignment (see
        wxHtmlContainerCell::SetAlignHor).
        """

    def SetContainer(self, c):
        """
        SetContainer(c) -> HtmlContainerCell
        
        Allows you to directly set opened container.
        """

    def SetDC(self, dc, pixel_scale=1.0e+0):
        """
        SetDC(dc, pixel_scale=1.0e+0)
        
        Sets the DC.
        """

    def SetFontBold(self, x):
        """
        SetFontBold(x)
        
        Sets bold flag of actualfont.
        """

    def SetFontFace(self, face):
        """
        SetFontFace(face)
        
        Sets current font face to face.
        """

    def SetFontFixed(self, x):
        """
        SetFontFixed(x)
        
        Sets fixed face flag of actualfont.
        """

    def SetFontItalic(self, x):
        """
        SetFontItalic(x)
        
        Sets italic flag of actualfont.
        """

    def SetFontSize(self, s):
        """
        SetFontSize(s)
        
        Sets actual font size (HTML size varies from 1 to 7).
        """

    def SetFontUnderlined(self, x):
        """
        SetFontUnderlined(x)
        
        Sets underlined flag of actualfont.
        """

    def SetFonts(self, normal_face, fixed_face, sizes):
        """
        SetFonts(normal_face, fixed_face, sizes)
        
        Sets fonts.
        """

    def SetLink(self, link):
        """
        SetLink(link)
        
        Sets actual hypertext link.
        """

    def SetLinkColor(self, clr):
        """
        SetLinkColor(clr)
        
        Sets colour of hypertext link.
        """
    ActualColor = property(None, None)
    Align = property(None, None)
    CharHeight = property(None, None)
    CharWidth = property(None, None)
    Container = property(None, None)
    DC = property(None, None)
    FontBold = property(None, None)
    FontFace = property(None, None)
    FontFixed = property(None, None)
    FontItalic = property(None, None)
    FontSize = property(None, None)
    FontUnderlined = property(None, None)
    Link = property(None, None)
    LinkColor = property(None, None)
    WindowInterface = property(None, None)
# end of class HtmlWinParser


def HtmlWinParser_AddTagHandler(self, tagHandlerClass):
    """
    HtmlWinParser_AddTagHandler(tagHandlerClass)
    """
#-- end-htmlwinpars --#
#-- begin-helpdata --#

class HtmlBookRecord(object):
    """
    HtmlBookRecord(bookfile, basepath, title, start)
    
    Helper class for wxHtmlHelpData.
    """

    def __init__(self, bookfile, basepath, title, start):
        """
        HtmlBookRecord(bookfile, basepath, title, start)
        
        Helper class for wxHtmlHelpData.
        """

    def GetBookFile(self):
        """
        GetBookFile() -> String
        """

    def GetTitle(self):
        """
        GetTitle() -> String
        """

    def GetStart(self):
        """
        GetStart() -> String
        """

    def GetBasePath(self):
        """
        GetBasePath() -> String
        """

    def SetContentsRange(self, start, end):
        """
        SetContentsRange(start, end)
        """

    def GetContentsStart(self):
        """
        GetContentsStart() -> int
        """

    def GetContentsEnd(self):
        """
        GetContentsEnd() -> int
        """

    def SetTitle(self, title):
        """
        SetTitle(title)
        """

    def SetBasePath(self, path):
        """
        SetBasePath(path)
        """

    def SetStart(self, start):
        """
        SetStart(start)
        """

    def GetFullPath(self, page):
        """
        GetFullPath(page) -> String
        """
    BasePath = property(None, None)
    BookFile = property(None, None)
    ContentsEnd = property(None, None)
    ContentsStart = property(None, None)
    Start = property(None, None)
    Title = property(None, None)
# end of class HtmlBookRecord


class HtmlHelpDataItem(object):
    """
    HtmlHelpDataItem()
    
    Helper class for wxHtmlHelpData.
    """

    def __init__(self):
        """
        HtmlHelpDataItem()
        
        Helper class for wxHtmlHelpData.
        """
    level = property(None, None)
    parent = property(None, None)
    id = property(None, None)
    name = property(None, None)
    page = property(None, None)
    book = property(None, None)

    def GetFullPath(self):
        """
        GetFullPath() -> String
        """

    def GetIndentedName(self):
        """
        GetIndentedName() -> String
        """
    FullPath = property(None, None)
    IndentedName = property(None, None)
# end of class HtmlHelpDataItem


class HtmlHelpData(wx.Object):
    """
    HtmlHelpData()
    
    This class is used by wxHtmlHelpController and wxHtmlHelpFrame to
    access HTML help items.
    """

    def __init__(self):
        """
        HtmlHelpData()
        
        This class is used by wxHtmlHelpController and wxHtmlHelpFrame to
        access HTML help items.
        """

    def AddBook(self, book_url):
        """
        AddBook(book_url) -> bool
        
        Adds new book.
        """

    def FindPageById(self, id):
        """
        FindPageById(id) -> String
        
        Returns page's URL based on integer ID stored in project.
        """

    def FindPageByName(self, page):
        """
        FindPageByName(page) -> String
        
        Returns page's URL based on its (file)name.
        """

    def GetBookRecArray(self):
        """
        GetBookRecArray() -> HtmlBookRecArray
        
        Returns array with help books info.
        """

    def GetContentsArray(self):
        """
        GetContentsArray() -> HtmlHelpDataItems
        
        Returns reference to array with contents entries.
        """

    def GetIndexArray(self):
        """
        GetIndexArray() -> HtmlHelpDataItems
        
        Returns reference to array with index entries.
        """

    def SetTempDir(self, path):
        """
        SetTempDir(path)
        
        Sets the temporary directory where binary cached versions of MS HTML
        Workshop files will be stored.
        """
    BookRecArray = property(None, None)
    ContentsArray = property(None, None)
    IndexArray = property(None, None)
# end of class HtmlHelpData

#-- end-helpdata --#
#-- begin-helpfrm --#
HF_TOOLBAR = 0
HF_CONTENTS = 0
HF_INDEX = 0
HF_SEARCH = 0
HF_BOOKMARKS = 0
HF_OPEN_FILES = 0
HF_PRINT = 0
HF_FLAT_TOOLBAR = 0
HF_MERGE_BOOKS = 0
HF_ICONS_BOOK = 0
HF_ICONS_BOOK_CHAPTER = 0
HF_ICONS_FOLDER = 0
HF_DEFAULT_STYLE = 0

class HtmlHelpFrame(wx.Frame):
    """
    HtmlHelpFrame(data=None)
    HtmlHelpFrame(parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE, data=None, config=None, rootpath=wx.EmptyString)
    
    This class is used by wxHtmlHelpController to display help.
    """

    def __init__(self, *args, **kw):
        """
        HtmlHelpFrame(data=None)
        HtmlHelpFrame(parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE, data=None, config=None, rootpath=wx.EmptyString)
        
        This class is used by wxHtmlHelpController to display help.
        """

    def AddToolbarButtons(self, toolBar, style):
        """
        AddToolbarButtons(toolBar, style)
        
        You may override this virtual method to add more buttons to the help
        window's toolbar.
        """

    def Create(self, parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE, config=None, rootpath=wx.EmptyString):
        """
        Create(parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE, config=None, rootpath=wx.EmptyString) -> bool
        
        Creates the frame.
        """

    def GetController(self):
        """
        GetController() -> HtmlHelpController
        
        Returns the help controller associated with the frame.
        """

    def SetController(self, controller):
        """
        SetController(controller)
        
        Sets the help controller associated with the frame.
        """

    def SetTitleFormat(self, format):
        """
        SetTitleFormat(format)
        
        Sets the frame's title format.
        """
    Controller = property(None, None)
# end of class HtmlHelpFrame

#-- end-helpfrm --#
#-- begin-helpdlg --#

class HtmlHelpDialog(wx.Dialog):
    """
    HtmlHelpDialog(data=None)
    HtmlHelpDialog(parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE, data=None)
    
    This class is used by wxHtmlHelpController to display help.
    """

    def __init__(self, *args, **kw):
        """
        HtmlHelpDialog(data=None)
        HtmlHelpDialog(parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE, data=None)
        
        This class is used by wxHtmlHelpController to display help.
        """

    def AddToolbarButtons(self, toolBar, style):
        """
        AddToolbarButtons(toolBar, style)
        
        You may override this virtual method to add more buttons to the help
        window's toolbar.
        """

    def Create(self, parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE):
        """
        Create(parent, id=wx.ID_ANY, title=wx.EmptyString, style=HF_DEFAULT_STYLE) -> bool
        
        Creates the dialog.
        """

    def GetController(self):
        """
        GetController() -> HtmlHelpController
        
        Returns the help controller associated with the dialog.
        """

    def SetController(self, controller):
        """
        SetController(controller)
        
        Sets the help controller associated with the dialog.
        """

    def SetTitleFormat(self, format):
        """
        SetTitleFormat(format)
        
        Sets the dialog's title format.
        """
    Controller = property(None, None)
# end of class HtmlHelpDialog

#-- end-helpdlg --#
#-- begin-helpwnd --#
ID_HTML_PANEL = 0
ID_HTML_BACK = 0
ID_HTML_FORWARD = 0
ID_HTML_UPNODE = 0
ID_HTML_UP = 0
ID_HTML_DOWN = 0
ID_HTML_PRINT = 0
ID_HTML_OPENFILE = 0
ID_HTML_OPTIONS = 0
ID_HTML_BOOKMARKSLIST = 0
ID_HTML_BOOKMARKSADD = 0
ID_HTML_BOOKMARKSREMOVE = 0
ID_HTML_TREECTRL = 0
ID_HTML_INDEXPAGE = 0
ID_HTML_INDEXLIST = 0
ID_HTML_INDEXTEXT = 0
ID_HTML_INDEXBUTTON = 0
ID_HTML_INDEXBUTTONALL = 0
ID_HTML_NOTEBOOK = 0
ID_HTML_SEARCHPAGE = 0
ID_HTML_SEARCHTEXT = 0
ID_HTML_SEARCHLIST = 0
ID_HTML_SEARCHBUTTON = 0
ID_HTML_SEARCHCHOICE = 0
ID_HTML_COUNTINFO = 0

class HtmlHelpWindow(wx.Window):
    """
    HtmlHelpWindow(data=None)
    HtmlHelpWindow(parent, WindowID, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL|wx.BORDER_NONE, helpStyle=HF_DEFAULT_STYLE, data=None)
    
    This class is used by wxHtmlHelpController to display help within a
    frame or dialog, but you can use it yourself to create an embedded
    HTML help window.
    """

    def __init__(self, *args, **kw):
        """
        HtmlHelpWindow(data=None)
        HtmlHelpWindow(parent, WindowID, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL|wx.BORDER_NONE, helpStyle=HF_DEFAULT_STYLE, data=None)
        
        This class is used by wxHtmlHelpController to display help within a
        frame or dialog, but you can use it yourself to create an embedded
        HTML help window.
        """

    def Create(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL|wx.BORDER_NONE, helpStyle=HF_DEFAULT_STYLE):
        """
        Create(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL|wx.BORDER_NONE, helpStyle=HF_DEFAULT_STYLE) -> bool
        
        Creates the help window.
        """

    def Display(self, *args, **kw):
        """
        Display(x) -> bool
        Display(id) -> bool
        
        Displays page x.
        """

    def DisplayContents(self):
        """
        DisplayContents() -> bool
        
        Displays contents panel.
        """

    def DisplayIndex(self):
        """
        DisplayIndex() -> bool
        
        Displays index panel.
        """

    def GetData(self):
        """
        GetData() -> HtmlHelpData
        
        Returns the wxHtmlHelpData object, which is usually a pointer to the
        controller's data.
        """

    def KeywordSearch(self, keyword, mode=wx.HELP_SEARCH_ALL):
        """
        KeywordSearch(keyword, mode=wx.HELP_SEARCH_ALL) -> bool
        
        Search for given keyword.
        """

    def ReadCustomization(self, cfg, path=wx.EmptyString):
        """
        ReadCustomization(cfg, path=wx.EmptyString)
        
        Reads the user's settings for this window.
        """

    def UseConfig(self, config, rootpath=wx.EmptyString):
        """
        UseConfig(config, rootpath=wx.EmptyString)
        
        Associates a wxConfig object with the help window.
        """

    def WriteCustomization(self, cfg, path=wx.EmptyString):
        """
        WriteCustomization(cfg, path=wx.EmptyString)
        
        Saves the user's settings for this window.
        """

    def RefreshLists(self):
        """
        RefreshLists()
        
        Refresh all panels.
        """

    def GetController(self):
        """
        GetController() -> HtmlHelpController
        """

    def SetController(self, controller):
        """
        SetController(controller)
        """
    Controller = property(None, None)
    Data = property(None, None)

    def CreateSearch(self):
        """
        CreateSearch()
        
        Creates search panel.
        """

    def AddToolbarButtons(self, toolBar, style):
        """
        AddToolbarButtons(toolBar, style)
        
        You may override this virtual method to add more buttons to the help
        window's toolbar.
        """

    def CreateContents(self):
        """
        CreateContents()
        
        Creates contents panel.
        """

    def CreateIndex(self):
        """
        CreateIndex()
        
        Creates index panel.
        """
# end of class HtmlHelpWindow

#-- end-helpwnd --#
#-- begin-helpctrl --#
ID_HTML_HELPFRAME = 0
HF_EMBEDDED = 0
HF_DIALOG = 0
HF_FRAME = 0
HF_MODAL = 0

class HtmlHelpController(wx.HelpControllerBase):
    """
    HtmlHelpController(style=HF_DEFAULT_STYLE, parentWindow=None)
    HtmlHelpController(parentWindow, style=HF_DEFAULT_STYLE)
    
    This help controller provides an easy way of displaying HTML help in
    your application (see HTML Sample, test example).
    """

    def __init__(self, *args, **kw):
        """
        HtmlHelpController(style=HF_DEFAULT_STYLE, parentWindow=None)
        HtmlHelpController(parentWindow, style=HF_DEFAULT_STYLE)
        
        This help controller provides an easy way of displaying HTML help in
        your application (see HTML Sample, test example).
        """

    def AddBook(self, *args, **kw):
        """
        AddBook(bookFile, showWaitMsg=False) -> bool
        AddBook(bookUrl, showWaitMsg=False) -> bool
        
        Adds a book (i.e.
        """

    def Display(self, *args, **kw):
        """
        Display(x) -> bool
        Display(id) -> bool
        
        Displays page x.
        """

    def DisplayContents(self):
        """
        DisplayContents() -> bool
        
        Displays help window and focuses contents panel.
        """

    def DisplayIndex(self):
        """
        DisplayIndex() -> bool
        
        Displays help window and focuses index panel.
        """

    def KeywordSearch(self, keyword, mode=wx.HELP_SEARCH_ALL):
        """
        KeywordSearch(keyword, mode=wx.HELP_SEARCH_ALL) -> bool
        
        Displays the help window, focuses search panel and starts searching.
        """

    def ReadCustomization(self, cfg, path=wx.EmptyString):
        """
        ReadCustomization(cfg, path=wx.EmptyString)
        
        Reads the controller's setting (position of window, etc.)
        """

    def SetShouldPreventAppExit(self, enable):
        """
        SetShouldPreventAppExit(enable)
        
        Sets whether the help frame should prevent application from exiting if
        it's the only remaining top level window.
        """

    def SetTempDir(self, path):
        """
        SetTempDir(path)
        
        Sets the path for storing temporary files - cached binary versions of
        index and contents files.
        """

    def SetTitleFormat(self, format):
        """
        SetTitleFormat(format)
        
        Sets format of title of the frame.
        """

    def UseConfig(self, config, rootpath=wx.EmptyString):
        """
        UseConfig(config, rootpath=wx.EmptyString)
        
        Associates the config object with the controller.
        """

    def WriteCustomization(self, cfg, path=wx.EmptyString):
        """
        WriteCustomization(cfg, path=wx.EmptyString)
        
        Stores controllers setting (position of window etc.)
        """

    def GetHelpWindow(self):
        """
        GetHelpWindow() -> HtmlHelpWindow
        
        Get the current help window.
        """

    def SetHelpWindow(self, helpWindow):
        """
        SetHelpWindow(helpWindow)
        
        Set the help window to be managed by this controller.
        """

    def GetFrame(self):
        """
        GetFrame() -> HtmlHelpFrame
        
        Returns the current help frame.
        """

    def GetDialog(self):
        """
        GetDialog() -> HtmlHelpDialog
        
        Returns the current help dialog.
        """
    Dialog = property(None, None)
    Frame = property(None, None)
    HelpWindow = property(None, None)

    def CreateHelpDialog(self, data):
        """
        CreateHelpDialog(data) -> HtmlHelpDialog
        
        This protected virtual method may be overridden so that when
        specifying the wxHF_DIALOG style, the controller uses a different
        dialog.
        """

    def CreateHelpFrame(self, data):
        """
        CreateHelpFrame(data) -> HtmlHelpFrame
        
        This protected virtual method may be overridden so that the controller
        uses a different frame.
        """
# end of class HtmlHelpController


class HtmlModalHelp(object):
    """
    HtmlModalHelp(parent, helpFile, topic=wx.EmptyString, style=HF_DEFAULT_STYLE|HF_DIALOG|HF_MODAL)
    
    This class uses wxHtmlHelpController to display help in a modal
    dialog.
    """

    def __init__(self, parent, helpFile, topic=wx.EmptyString, style=HF_DEFAULT_STYLE|HF_DIALOG|HF_MODAL):
        """
        HtmlModalHelp(parent, helpFile, topic=wx.EmptyString, style=HF_DEFAULT_STYLE|HF_DIALOG|HF_MODAL)
        
        This class uses wxHtmlHelpController to display help in a modal
        dialog.
        """
# end of class HtmlModalHelp

#-- end-helpctrl --#
#-- begin-htmllbox --#
HLB_DEFAULT_STYLE = 0
HLB_MULTIPLE = 0
HtmlListBoxNameStr = ""

class HtmlListBox(wx.VListBox):
    """
    HtmlListBox(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, name=HtmlListBoxNameStr)
    HtmlListBox()
    
    wxHtmlListBox is an implementation of wxVListBox which shows HTML
    content in the listbox rows.
    """

    def __init__(self, *args, **kw):
        """
        HtmlListBox(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, name=HtmlListBoxNameStr)
        HtmlListBox()
        
        wxHtmlListBox is an implementation of wxVListBox which shows HTML
        content in the listbox rows.
        """

    def GetFileSystem(self):
        """
        GetFileSystem() -> wx.FileSystem
        
        Returns the wxFileSystem used by the HTML parser of this object.
        """

    def Create(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, name=HtmlListBoxNameStr):
        """
        Create(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, name=HtmlListBoxNameStr) -> bool
        
        Creates the control and optionally sets the initial number of items in
        it (it may also be set or changed later with
        wxVListBox::SetItemCount).
        """
    FileSystem = property(None, None)

    def OnLinkClicked(self, n, link):
        """
        OnLinkClicked(n, link)
        
        Called when the user clicks on hypertext link.
        """

    def GetSelectedTextBgColour(self, colBg):
        """
        GetSelectedTextBgColour(colBg) -> wx.Colour
        
        This virtual function may be overridden to change the appearance of
        the background of the selected cells in the same way as
        GetSelectedTextColour().
        """

    def GetSelectedTextColour(self, colFg):
        """
        GetSelectedTextColour(colFg) -> wx.Colour
        
        This virtual function may be overridden to customize the appearance of
        the selected cells.
        """

    def OnGetItemMarkup(self, n):
        """
        OnGetItemMarkup(n) -> String
        
        This function may be overridden to decorate HTML returned by
        OnGetItem().
        """

    def OnGetItem(self, n):
        """
        OnGetItem(n) -> String
        
        This method must be implemented in the derived class and should return
        the body (i.e.
        """
# end of class HtmlListBox

SimpleHtmlListBoxNameStr = ""

class SimpleHtmlListBox(HtmlListBox, wx.ItemContainer):
    """
    SimpleHtmlListBox(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, choices=[], style=HLB_DEFAULT_STYLE, validator=wx.DefaultValidator, name=SimpleHtmlListBoxNameStr)
    SimpleHtmlListBox()
    
    wxSimpleHtmlListBox is an implementation of wxHtmlListBox which shows
    HTML content in the listbox rows.
    """

    def __init__(self, *args, **kw):
        """
        SimpleHtmlListBox(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, choices=[], style=HLB_DEFAULT_STYLE, validator=wx.DefaultValidator, name=SimpleHtmlListBoxNameStr)
        SimpleHtmlListBox()
        
        wxSimpleHtmlListBox is an implementation of wxHtmlListBox which shows
        HTML content in the listbox rows.
        """

    def Create(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, choices=[], style=HLB_DEFAULT_STYLE, validator=wx.DefaultValidator, name=SimpleHtmlListBoxNameStr):
        """
        Create(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, choices=[], style=HLB_DEFAULT_STYLE, validator=wx.DefaultValidator, name=SimpleHtmlListBoxNameStr) -> bool
        
        Creates the HTML listbox for two-step construction.
        """
# end of class SimpleHtmlListBox

#-- end-htmllbox --#



#----------------------------------------------------------------------------
# Name:        wx.lib.inspection
# Purpose:     A widget inspection tool that allows easy introspection of
#              all the live widgets and sizers in an application.
#
# Author:      Robin Dunn
#
# Created:     26-Jan-2007
# Copyright:   (c) 2007-2018 by Total Control Software
# Licence:     wxWindows license
#
# Tags:        py3-port, phoenix-port, documented
#----------------------------------------------------------------------------

# NOTE: This class was originally based on ideas sent to the
# wxPython-users mail list by Dan Eloff.  See also
# wx.lib.mixins.inspect for a class that can be mixed-in with wx.App
# to provide Hot-Key access to the inspection tool.

"""
This modules provides the :class:`~wx.lib.inspection.InspectionTool` and
everything else needed to provide the Widget Inspection Tool (WIT).
"""


import wx
import wx.py
import wx.stc
#import wx.aui as aui
import wx.lib.agw.aui as aui
import six
import wx.lib.utils as utils
import sys
import inspect

#----------------------------------------------------------------------------

class InspectionTool:
    """
    The :class:`InspectionTool` is a singleton that manages creating and
    showing an :class:`InspectionFrame`.
    """

    # Note: This is the Borg design pattern which ensures that all
    # instances of this class are actually using the same set of
    # instance data.  See
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66531
    __shared_state = None

    def __init__(self):
        if not InspectionTool.__shared_state:
            InspectionTool.__shared_state = self.__dict__
        else:
            self.__dict__ = InspectionTool.__shared_state

        if not hasattr(self, 'initialized'):
            self.initialized = False


    def Init(self, pos=wx.DefaultPosition, size=wx.Size(850,700),
             config=None, locals=None, app=None):
        """
        Init is used to set some parameters that will be used later
        when the inspection tool is shown.  Suitable defaults will be
        used for all of these parameters if they are not provided.

        :param `pos`:   The default position to show the frame at
        :param `size`:  The default size of the frame
        :param `config`: A :class:`Config` object to be used to store layout
            and other info to when the inspection frame is closed.
            This info will be restored the next time the inspection
            frame is used.
        :param `locals`: A dictionary of names to be added to the PyCrust
            namespace.
        :param `app`:  A reference to the :class:`App` object.
        """
        self._frame = None
        self._pos = pos
        self._size = size
        self._config = config
        self._locals = locals
        self._app = app
        if not self._app:
            self._app = wx.GetApp()
        self.initialized = True


    def Show(self, selectObj=None, refreshTree=False):
        """
        Creates the inspection frame if it hasn't been already, and
        raises it if neccessary.

        :param `selectObj`: Pass a widget or sizer to have that object be
                     preselected in widget tree.
        :param boolean `refreshTree`: rebuild the widget tree, default False

        """
        if not self.initialized:
            self.Init()

        parent = self._app.GetTopWindow()
        if not selectObj:
            selectObj = parent
        if not self._frame:
            self._frame = InspectionFrame( parent=parent,
                                           pos=self._pos,
                                           size=self._size,
                                           config=self._config,
                                           locals=self._locals,
                                           app=self._app)
        if selectObj:
            self._frame.SetObj(selectObj)
        if refreshTree:
            self._frame.RefreshTree()
        self._frame.Show()
        if self._frame.IsIconized():
            self._frame.Iconize(False)
        self._frame.Raise()


#----------------------------------------------------------------------------


class InspectionFrame(wx.Frame):
    """
    This class is the frame that holds the wxPython inspection tools.
    The toolbar and AUI splitters/floating panes are also managed
    here.  The contents of the tool windows are handled by other
    classes.
    """
    def __init__(self, wnd=None, locals=None, config=None,
                 app=None, title="wxPython Widget Inspection Tool",
                 *args, **kw):
        kw['title'] = title
        wx.Frame.__init__(self, *args, **kw)

        self.SetExtraStyle(wx.WS_EX_BLOCK_EVENTS)
        self.includeSizers = False
        self.started = False

        self.SetIcon(Icon.GetIcon())
        self.MakeToolBar()
        panel = wx.Panel(self, size=self.GetClientSize())

        # tell FrameManager to manage this frame
        self.mgr = aui.AuiManager(panel,
                                  aui.AUI_MGR_DEFAULT
                                  | aui.AUI_MGR_TRANSPARENT_DRAG
                                  | aui.AUI_MGR_ALLOW_ACTIVE_PANE)

        # make the child tools
        self.tree = InspectionTree(panel, size=(100,300))
        self.info = InspectionInfoPanel(panel,
                                        style=wx.NO_BORDER,
                                        )

        if not locals:
            locals = {}
        myIntroText = (
            "Python %s on %s, wxPython %s\n"
            "NOTE: The 'obj' variable refers to the object selected in the tree."
            % (sys.version.split()[0], sys.platform, wx.version()))
        self.crust = wx.py.crust.Crust(panel, locals=locals,
                                       intro=myIntroText,
                                       showInterpIntro=False,
                                       style=wx.NO_BORDER,
                                       )
        self.locals = self.crust.shell.interp.locals
        self.crust.shell.interp.introText = ''
        self.locals['obj'] = self.obj = wnd
        self.locals['app'] = app
        self.locals['wx'] = wx
        wx.CallAfter(self._postStartup)

        # put the chlid tools in AUI panes
        self.mgr.AddPane(self.info,
                         aui.AuiPaneInfo().Name("info").Caption("Object Info").
                         CenterPane().CaptionVisible(True).
                         CloseButton(False).MaximizeButton(True)
                         )
        self.mgr.AddPane(self.tree,
                         aui.AuiPaneInfo().Name("tree").Caption("Widget Tree").
                         CaptionVisible(True).Left().Dockable(True).Floatable(True).
                         BestSize((280,200)).CloseButton(False).MaximizeButton(True)
                         )
        self.mgr.AddPane(self.crust,
                         aui.AuiPaneInfo().Name("crust").Caption("PyCrust").
                         CaptionVisible(True).Bottom().Dockable(True).Floatable(True).
                         BestSize((400,200)).CloseButton(False).MaximizeButton(True)
                         )

        self.mgr.Update()

        if config is None:
            config = wx.Config('wxpyinspector')
        self.config = config
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        if self.Parent:
            tlw = self.Parent.GetTopLevelParent()
            tlw.Bind(wx.EVT_CLOSE, self.OnClose)
        self.LoadSettings(self.config)
        self.crust.shell.lineNumbers = False
        self.crust.shell.setDisplayLineNumbers(False)
        self.crust.shell.SetMarginWidth(1, 0)


    def MakeToolBar(self):
        tbar = self.CreateToolBar(wx.TB_HORIZONTAL | wx.TB_FLAT | wx.TB_TEXT | wx.NO_BORDER )
        tbar.SetToolBitmapSize((24,24))

        refreshBmp = Refresh.GetBitmap()
        findWidgetBmp = Find.GetBitmap()
        showSizersBmp = ShowSizers.GetBitmap()
        expandTreeBmp = ExpandTree.GetBitmap()
        collapseTreeBmp = CollapseTree.GetBitmap()
        highlightItemBmp = HighlightItem.GetBitmap()
        evtWatcherBmp = EvtWatcher.GetBitmap()

        toggleFillingBmp = ShowFilling.GetBitmap()

        refreshTool = tbar.AddTool(-1, 'Refresh', refreshBmp,
                                   shortHelp = 'Refresh widget tree (F1)')
        findWidgetTool = tbar.AddTool(-1, 'Find', findWidgetBmp,
                                      shortHelp='Find new target widget. (F2)  Click here and\nthen on another widget in the app.')
        showSizersTool = tbar.AddTool(-1, 'Sizers', showSizersBmp,
                                      shortHelp='Include sizers in widget tree (F3)',
                                      kind=wx.ITEM_CHECK)
        expandTreeTool = tbar.AddTool(-1, 'Expand', expandTreeBmp,
                                      shortHelp='Expand all tree items (F4)')
        collapseTreeTool = tbar.AddTool(-1, 'Collapse', collapseTreeBmp,
                                        shortHelp='Collapse all tree items (F5)')
        highlightItemTool = tbar.AddTool(-1, 'Highlight', highlightItemBmp,
                                         shortHelp='Attempt to highlight live item (F6)')
        evtWatcherTool = tbar.AddTool(-1, 'Events', evtWatcherBmp,
                                      shortHelp='Watch the events of the selected item (F7)')

        toggleFillingTool = tbar.AddTool(-1, 'Filling', toggleFillingBmp,
                                         shortHelp='Show PyCrust \'filling\' (F8)',
                                         kind=wx.ITEM_CHECK)
        tbar.Realize()

        self.Bind(wx.EVT_TOOL,      self.OnRefreshTree,     refreshTool)
        self.Bind(wx.EVT_TOOL,      self.OnFindWidget,      findWidgetTool)
        self.Bind(wx.EVT_TOOL,      self.OnShowSizers,      showSizersTool)
        self.Bind(wx.EVT_TOOL,      self.OnExpandTree,      expandTreeTool)
        self.Bind(wx.EVT_TOOL,      self.OnCollapseTree,    collapseTreeTool)
        self.Bind(wx.EVT_TOOL,      self.OnHighlightItem,   highlightItemTool)
        self.Bind(wx.EVT_TOOL,      self.OnWatchEvents,     evtWatcherTool)
        self.Bind(wx.EVT_TOOL,      self.OnToggleFilling,   toggleFillingTool)
        self.Bind(wx.EVT_UPDATE_UI, self.OnShowSizersUI,    showSizersTool)
        self.Bind(wx.EVT_UPDATE_UI, self.OnWatchEventsUI,   evtWatcherTool)
        self.Bind(wx.EVT_UPDATE_UI, self.OnToggleFillingUI, toggleFillingTool)

        tbl = wx.AcceleratorTable(
            [(wx.ACCEL_NORMAL, wx.WXK_F1, refreshTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F2, findWidgetTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F3, showSizersTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F4, expandTreeTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F5, collapseTreeTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F6, highlightItemTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F7, evtWatcherTool.GetId()),
             (wx.ACCEL_NORMAL, wx.WXK_F8, toggleFillingTool.GetId()),
             ])
        self.SetAcceleratorTable(tbl)


    def _postStartup(self):
        if self.crust.ToolsShown():
            self.crust.ToggleTools()
        self.UpdateInfo()
        self.started = True


    def OnClose(self, evt):
        evt.Skip()
        if not self:
            return
        self.SaveSettings(self.config)
        if hasattr(self, 'mgr'):
            self.mgr.UnInit()
            del self.mgr
        if self.Parent:
            tlw = self.Parent.GetTopLevelParent()
            tlw.Unbind(wx.EVT_CLOSE, handler=self.OnClose)


    def UpdateInfo(self):
        self.info.UpdateInfo(self.obj)


    def SetObj(self, obj):
        if self.obj is obj:
            return
        self.locals['obj'] = self.obj = obj
        self.UpdateInfo()
        if not self.tree.built:
            self.tree.BuildTree(obj, includeSizers=self.includeSizers)
        else:
            self.tree.SelectObj(obj)


    def HighlightCurrentItem(self):
        """
        Draw a highlight rectangle around the item represented by the
        current tree selection.
        """
        if not hasattr(self, 'highlighter'):
            self.highlighter = _InspectionHighlighter()
        self.highlighter.HighlightCurrentItem(self.tree)


    def RefreshTree(self):
        self.tree.BuildTree(self.obj, includeSizers=self.includeSizers)


    def OnRefreshTree(self, evt):
        self.RefreshTree()
        self.UpdateInfo()


    def OnFindWidget(self, evt):
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_MOUSE_CAPTURE_LOST, self.OnCaptureLost)
        self.CaptureMouse()
        self.finding = wx.BusyInfo("Click on any widget in the app...")


    def OnCaptureLost(self, evt):
        self.Unbind(wx.EVT_LEFT_DOWN)
        self.Unbind(wx.EVT_MOUSE_CAPTURE_LOST)
        del self.finding

    def OnLeftDown(self, evt):
        self.ReleaseMouse()
        wnd, pt = wx.FindWindowAtPointer()
        if wnd is not None:
            self.SetObj(wnd)
        else:
            wx.Bell()
        self.OnCaptureLost(evt)


    def OnShowSizers(self, evt):
        self.includeSizers = not self.includeSizers
        self.RefreshTree()


    def OnExpandTree(self, evt):
        current = self.tree.GetSelection()
        self.tree.ExpandAll()
        self.tree.EnsureVisible(current)


    def OnCollapseTree(self, evt):
        current = self.tree.GetSelection()
        self.tree.CollapseAll()
        self.tree.EnsureVisible(current)
        self.tree.SelectItem(current)


    def OnHighlightItem(self, evt):
        self.HighlightCurrentItem()


    def OnWatchEvents(self, evt):
        item = self.tree.GetSelection()
        obj = self.tree.GetItemData(item)
        if isinstance(obj, wx.Window):
            import wx.lib.eventwatcher as ew
            watcher = ew.EventWatcher(self)
            watcher.watch(obj)
            watcher.Show()

    def OnWatchEventsUI(self, evt):
        item = self.tree.GetSelection()
        if item:
            obj = self.tree.GetItemData(item)
            evt.Enable(isinstance(obj, wx.Window))


    def OnToggleFilling(self, evt):
        self.crust.ToggleTools()


    def OnShowSizersUI(self, evt):
        evt.Check(self.includeSizers)


    def OnToggleFillingUI(self, evt):
        if self.started:
            evt.Check(self.crust.ToolsShown())


    def LoadSettings(self, config):
        self.crust.LoadSettings(config)
        self.info.LoadSettings(config)

        pos  = wx.Point(config.ReadInt('Window/PosX', -1),
                        config.ReadInt('Window/PosY', -1))

        size = wx.Size(config.ReadInt('Window/Width', -1),
                       config.ReadInt('Window/Height', -1))
        self.SetSize(size)
        self.Move(pos)
        rect = utils.AdjustRectToScreen(self.GetRect())
        self.SetRect(rect)

        perspective = config.Read('perspective', '')
        if perspective:
            try:
                self.mgr.LoadPerspective(perspective)
            except wx.PyAssertionError:
                # ignore bad perspective string errors
                pass
        self.includeSizers = config.ReadBool('includeSizers', False)


    def SaveSettings(self, config):
        self.crust.SaveSettings(config)
        self.info.SaveSettings(config)

        if not self.IsIconized() and not self.IsMaximized():
            w, h = self.GetSize()
            config.WriteInt('Window/Width', w)
            config.WriteInt('Window/Height', h)

            px, py = self.GetPosition()
            config.WriteInt('Window/PosX', px)
            config.WriteInt('Window/PosY', py)

        if hasattr(self, "mgr"):
            perspective = self.mgr.SavePerspective()
            config.Write('perspective', perspective)
            config.WriteBool('includeSizers', self.includeSizers)

#---------------------------------------------------------------------------

# should inspection frame (and children) be includeed in the tree?
INCLUDE_INSPECTOR = True

USE_CUSTOMTREECTRL = False
if USE_CUSTOMTREECTRL:
    import wx.lib.agw.customtreectrl as CT
    TreeBaseClass = CT.CustomTreeCtrl
else:
    TreeBaseClass = wx.TreeCtrl

class InspectionTree(TreeBaseClass):
    """
    All of the widgets in the app, and optionally their sizers, are
    loaded into this tree.
    """
    def __init__(self, *args, **kw):
        #s = kw.get('style', 0)
        #kw['style'] = s | wx.TR_DEFAULT_STYLE | wx.TR_HIDE_ROOT
        TreeBaseClass.__init__(self, *args, **kw)
        self.roots = []
        self.built = False
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelectionChanged)
        self.toolFrame = wx.GetTopLevelParent(self)
        if 'wxMac' in wx.PlatformInfo:
            self.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)


    def BuildTree(self, startWidget, includeSizers=False, expandFrame=False):
        if self.GetCount():
            self.DeleteAllItems()
            self.roots = []
            self.built = False

        realRoot = self.AddRoot('Top-level Windows')

        for w in wx.GetTopLevelWindows():
            if w is wx.GetTopLevelParent(self) and not INCLUDE_INSPECTOR:
                continue
            root  = self._AddWidget(realRoot, w, includeSizers)
            self.roots.append(root)

        # Expand the subtree containing the startWidget, and select it.
        if not startWidget or not isinstance(startWidget, wx.Window):
            startWidget = wx.GetApp().GetTopWindow()
        if expandFrame:
            top = wx.GetTopLevelParent(startWidget)
            topItem = self.FindWidgetItem(top)
            if topItem:
                self.ExpandAllChildren(topItem)
        self.built = True
        self.SelectObj(startWidget)


    def _AddWidget(self, parentItem, widget, includeSizers):
        text = self.GetTextForWidget(widget)
        item = self.AppendItem(parentItem, text)
        self.SetItemData(item, widget)

        # Add the sizer and widgets in the sizer, if we're showing them
        widgetsInSizer = []
        if includeSizers and widget.GetSizer() is not None:
            widgetsInSizer = self._AddSizer(item, widget.GetSizer())

        # Add any children not in the sizer, or all children if we're
        # not showing the sizers
        for child in widget.GetChildren():
            if (not child in widgetsInSizer and
                (not child.IsTopLevel() or
                 isinstance(child, wx.PopupWindow))):
                self._AddWidget(item, child, includeSizers)

        return item


    def _AddSizer(self, parentItem, sizer):
        widgets = []
        text = self.GetTextForSizer(sizer)
        item = self.AppendItem(parentItem, text)
        self.SetItemData(item, sizer)
        self.SetItemTextColour(item, "blue")

        for si in sizer.GetChildren():
            if si.IsWindow():
                w = si.GetWindow()
                self._AddWidget(item, w, True)
                widgets.append(w)
            elif si.IsSizer():
                ss = si.GetSizer()
                widgets += self._AddSizer(item, ss)
                ss._parentSizer = sizer
            else:
                i = self.AppendItem(item, "Spacer")
                self.SetItemData(i, si)
                self.SetItemTextColour(i, "blue")
        return widgets


    def FindWidgetItem(self, widget):
        """
        Find the tree item for a widget.
        """
        for item in self.roots:
            found = self._FindWidgetItem(widget, item)
            if found:
                return found
        return None

    def _FindWidgetItem(self, widget, item):
        if self.GetItemData(item) is widget:
            return item
        child, cookie = self.GetFirstChild(item)
        while child:
            found = self._FindWidgetItem(widget, child)
            if found:
                return found
            child, cookie = self.GetNextChild(item, cookie)
        return None


    def GetTextForWidget(self, widget):
        """
        Returns the string to be used in the tree for a widget
        """
        if hasattr(widget, 'GetName'):
            return "%s (\"%s\")" % (widget.__class__.__name__, widget.GetName())
        return widget.__class__.__name__


    def GetTextForSizer(self, sizer):
        """
        Returns the string to be used in the tree for a sizer
        """
        return "%s" % sizer.__class__.__name__


    def SelectObj(self, obj):
        item = self.FindWidgetItem(obj)
        if item:
            self.EnsureVisible(item)
            self.SelectItem(item)


    def OnSelectionChanged(self, evt):
        item = evt.GetItem()
        if item:
            obj = self.GetItemData(item)
            self.toolFrame.SetObj(obj)


#---------------------------------------------------------------------------

class InspectionInfoPanel(wx.stc.StyledTextCtrl):
    """
    Used to display information about the currently selected items.
    Currently just a read-only :class:`stc.StyledTextCtrl` with some plain
    text.  Should probably add some styles to make things easier to
    read.
    """
    def __init__(self, *args, **kw):
        wx.stc.StyledTextCtrl.__init__(self, *args, **kw)

        from wx.py.editwindow import FACES
        self.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT,
                          "face:%(mono)s,size:%(size)d,back:%(backcol)s" % FACES)
        self.StyleClearAll()
        self.SetReadOnly(True)
        self.SetMarginType(1, 0)
        self.SetMarginWidth(1, 0)
        self.SetSelForeground(True, wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT))
        self.SetSelBackground(True, wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT))


    def LoadSettings(self, config):
        zoom = config.ReadInt('View/Zoom/Info', 0)
        self.SetZoom(zoom)

    def SaveSettings(self, config):
        config.WriteInt('View/Zoom/Info', self.GetZoom())


    def UpdateInfo(self, obj):
        st = []
        if not obj:
            st.append("Item is None or has been destroyed.")

        elif isinstance(obj, wx.Window):
            st += self.FmtWidget(obj)

        elif isinstance(obj, wx.Sizer):
            st += self.FmtSizer(obj)

        elif isinstance(obj, wx.SizerItem):
            st += self.FmtSizerItem(obj)

        self.SetReadOnly(False)
        self.SetText('\n'.join(st))
        self.SetReadOnly(True)


    def Fmt(self, name, value):
        if isinstance(value, six.string_types):
            return "    %s = '%s'" % (name, value)
        else:
            return "    %s = %s" % (name, value)


    def FmtWidget(self, obj):
        def _countChildren(children):
            count = 0
            for child in children:
                if not child.IsTopLevel():
                    count += 1
                    count += _countChildren(child.GetChildren())
            return count
        def _countAllChildren(children):
            count = 0
            for child in children:
                count += 1
                count += _countAllChildren(child.GetChildren())
            return count

        count = len([c for c in obj.GetChildren() if not c.IsTopLevel()])
        rcount = _countChildren(obj.GetChildren())
        tlwcount = _countAllChildren(obj.GetChildren())

        st = ["Widget:"]
        if hasattr(obj, 'GetName'):
            st.append(self.Fmt('name',        obj.GetName()))
        st.append(self.Fmt('class',       obj.__class__))
        st.append(self.Fmt('bases',       obj.__class__.__bases__))
        st.append(self.Fmt('module',      inspect.getmodule(obj)))
        if hasattr(obj, 'this'):
            st.append(self.Fmt('this',    repr(obj.this)))
        st.append(self.Fmt('id',          obj.GetId()))
        st.append(self.Fmt('style',       obj.GetWindowStyle()))
        st.append(self.Fmt('pos',         obj.GetPosition()))
        st.append(self.Fmt('size',        obj.GetSize()))
        st.append(self.Fmt('minsize',     obj.GetMinSize()))
        st.append(self.Fmt('bestsize',    obj.GetBestSize()))
        st.append(self.Fmt('client size', obj.GetClientSize()))
        st.append(self.Fmt('virtual size',obj.GetVirtualSize()))
        st.append(self.Fmt('IsEnabled',   obj.IsEnabled()))
        st.append(self.Fmt('IsShown',     obj.IsShown()))
        st.append(self.Fmt('IsFrozen',    obj.IsFrozen()))
        st.append(self.Fmt('fg color',    obj.GetForegroundColour()))
        st.append(self.Fmt('bg color',    obj.GetBackgroundColour()))
        st.append(self.Fmt('label',       obj.GetLabel()))
        if hasattr(obj, 'GetTitle'):
            st.append(self.Fmt('title',   obj.GetTitle()))
        if hasattr(obj, 'GetValue'):
            try:
                st.append(self.Fmt('value',   obj.GetValue()))
            except Exception:
                pass
        st.append('    child count = %d (direct)  %d (recursive)  %d (include TLWs)' %
                  (count, rcount, tlwcount))
        if obj.GetContainingSizer() is not None:
            st.append('')
            sizer = obj.GetContainingSizer()
            st += self.FmtSizerItem(sizer.GetItem(obj))
        return st


    def FmtSizerItem(self, obj):
        if obj is None:
            return ['SizerItem: None']

        st = ['SizerItem:']
        st.append(self.Fmt('proportion', obj.GetProportion()))
        st.append(self.Fmt('flag',
                           FlagsFormatter(itemFlags, obj.GetFlag())))
        st.append(self.Fmt('border',     obj.GetBorder()))
        st.append(self.Fmt('pos',        obj.GetPosition()))
        st.append(self.Fmt('size',       obj.GetSize()))
        st.append(self.Fmt('minsize',    obj.GetMinSize()))
        st.append(self.Fmt('ratio',      obj.GetRatio()))
        st.append(self.Fmt('IsWindow',   obj.IsWindow()))
        st.append(self.Fmt('IsSizer',    obj.IsSizer()))
        st.append(self.Fmt('IsSpacer',   obj.IsSpacer()))
        st.append(self.Fmt('IsShown',    obj.IsShown()))
        if isinstance(obj, wx.GBSizerItem):
            st.append(self.Fmt('cellpos',    obj.GetPos()))
            st.append(self.Fmt('cellspan',   obj.GetSpan()))
            st.append(self.Fmt('endpos',     obj.GetEndPos()))
        return st


    def FmtSizer(self, obj):
        st = ['Sizer:']
        st.append(self.Fmt('class',      obj.__class__))
        if hasattr(obj, 'this'):
            st.append(self.Fmt('this',      repr(obj.this)))
        st.append(self.Fmt('pos',        obj.GetPosition()))
        st.append(self.Fmt('size',       obj.GetSize()))
        st.append(self.Fmt('minsize',    obj.GetMinSize()))
        if isinstance(obj, wx.BoxSizer):
            st.append(self.Fmt('orientation',
                               FlagsFormatter(orientFlags, obj.GetOrientation())))
        if isinstance(obj, wx.GridSizer):
            st.append(self.Fmt('cols', obj.GetCols()))
            st.append(self.Fmt('rows', obj.GetRows()))
            st.append(self.Fmt('vgap', obj.GetVGap()))
            st.append(self.Fmt('hgap', obj.GetHGap()))
        if isinstance(obj, wx.FlexGridSizer):
            st.append(self.Fmt('rowheights', obj.GetRowHeights()))
            st.append(self.Fmt('colwidths', obj.GetColWidths()))
            st.append(self.Fmt('flexdir',
                               FlagsFormatter(orientFlags, obj.GetFlexibleDirection())))
            st.append(self.Fmt('nonflexmode',
                               FlagsFormatter(flexmodeFlags, obj.GetNonFlexibleGrowMode())))
        if isinstance(obj, wx.GridBagSizer):
            st.append(self.Fmt('emptycell', obj.GetEmptyCellSize()))

        if hasattr(obj, '_parentSizer'):
            st.append('')
            st += self.FmtSizerItem(obj._parentSizer.GetItem(obj))

        return st


class FlagsFormatter(object):
    def __init__(self, d, val):
        self.d = d
        self.val = val

    def __str__(self):
        st = []
        for k in self.d.keys():
            if self.val & k:
                st.append(self.d[k])
        if st:
            return '|'.join(st)
        else:
            return '0'

orientFlags = {
    wx.HORIZONTAL : 'wx.HORIZONTAL',
    wx.VERTICAL : 'wx.VERTICAL',
    }

itemFlags = {
    wx.TOP : 'wx.TOP',
    wx.BOTTOM : 'wx.BOTTOM',
    wx.LEFT : 'wx.LEFT',
    wx.RIGHT : 'wx.RIGHT',
#    wx.ALL : 'wx.ALL',
    wx.EXPAND : 'wx.EXPAND',
#    wx.GROW : 'wx.GROW',
    wx.SHAPED : 'wx.SHAPED',
    wx.STRETCH_NOT : 'wx.STRETCH_NOT',
#    wx.ALIGN_CENTER : 'wx.ALIGN_CENTER',
    wx.ALIGN_LEFT : 'wx.ALIGN_LEFT',
    wx.ALIGN_RIGHT : 'wx.ALIGN_RIGHT',
    wx.ALIGN_TOP : 'wx.ALIGN_TOP',
    wx.ALIGN_BOTTOM : 'wx.ALIGN_BOTTOM',
    wx.ALIGN_CENTER_VERTICAL : 'wx.ALIGN_CENTER_VERTICAL',
    wx.ALIGN_CENTER_HORIZONTAL : 'wx.ALIGN_CENTER_HORIZONTAL',
    wx.ADJUST_MINSIZE : 'wx.ADJUST_MINSIZE',
    wx.FIXED_MINSIZE : 'wx.FIXED_MINSIZE',
    }

flexmodeFlags = {
    wx.FLEX_GROWMODE_NONE : 'wx.FLEX_GROWMODE_NONE',
    wx.FLEX_GROWMODE_SPECIFIED : 'wx.FLEX_GROWMODE_SPECIFIED',
    wx.FLEX_GROWMODE_ALL : 'wx.FLEX_GROWMODE_ALL',
    }




#---------------------------------------------------------------------------

class _InspectionHighlighter(object):
    """
    All the highlighting code.  A separate class to help reduce the
    clutter in InspectionFrame.
    """

    # should non TLWs be flashed too?  Otherwise use a highlight rectangle
    flashAll = False

    color1 = 'red'         # for widgets and sizers
    color2 = 'red'         # for item boundaries in sizers
    color3 = '#00008B'     # for items in sizers

    highlightTime = 3000   # how long to display the highlights

                           # how to draw it
    useOverlay = 'wxMac' in wx.PlatformInfo


    def __init__(self):
        if self.useOverlay:
            self.overlay = wx.Overlay()


    def HighlightCurrentItem(self, tree):
        """
        Draw a highlight rectangle around the item represented by the
        current tree selection.
        """
        item = tree.GetSelection()
        obj = tree.GetItemData(item)

        if isinstance(obj, wx.Window):
            self.HighlightWindow(obj)

        elif isinstance(obj, wx.Sizer):
            self.HighlightSizer(obj)

        elif isinstance(obj, wx.SizerItem):   # Spacer
            pItem = tree.GetItemParent(item)
            sizer = tree.GetItemData(pItem)
            self.HighlightSizerItem(obj, sizer)

        else:
            raise RuntimeError("unknown object type: %s" % obj.__class__.__name__)


    def HighlightWindow(self, win):
        rect = win.GetRect()
        tlw = win.GetTopLevelParent()
        if self.flashAll or tlw is win:
            self.FlickerTLW(win)
            return
        else:
            pos = self.FindHighlightPos(tlw, win.ClientToScreen((0,0)))
            rect.SetPosition(pos)
            self.DoHighlight(tlw, rect, self.color1)


    def HighlightSizerItem(self, item, sizer, penWidth=2):
        win = sizer.GetContainingWindow()
        tlw = win.GetTopLevelParent()
        rect = item.GetRect()
        pos = rect.GetPosition()
        pos = self.FindHighlightPos(tlw, win.ClientToScreen(pos))
        rect.SetPosition(pos)
        if rect.width < 1: rect.width = 1
        if rect.width < 1: rect.width = 1
        self.DoHighlight(tlw, rect, self.color1, penWidth)


    def HighlightSizer(self, sizer):
        # first do the outline of the whole sizer like normal
        win = sizer.GetContainingWindow()
        tlw = win.GetTopLevelParent()
        pos = sizer.GetPosition()
        pos = self.FindHighlightPos(tlw, win.ClientToScreen(pos))
        rect = wx.Rect(pos, sizer.GetSize())
        dc, dco = self.DoHighlight(tlw, rect, self.color1)

        # Now highlight the actual items within the sizer.  This may
        # get overdrawn by the code below for item boundaries, but if
        # there is border padding then this will help make it more
        # obvious.
        dc.SetPen(wx.Pen(self.color3, 1))
        for item in sizer.GetChildren():
            if item.IsShown():
                if item.IsWindow():
                    r = item.GetWindow().GetRect()
                elif item.IsSizer():
                    p = item.GetSizer().GetPosition()
                    s = item.GetSizer().GetSize()
                    r = wx.Rect(p,s)
                else:
                    continue
                r = self.AdjustRect(tlw, win, r)
                dc.DrawRectangle(r)

        # Next highlight the area allocated to each item in the sizer.
        # Each kind of sizer will need to be done a little
        # differently.
        dc.SetPen(wx.Pen(self.color2, 1))

        if isinstance(sizer, wx.WrapSizer):
            for item in sizer.GetChildren():
                ir = self.AdjustRect(tlw, win, item.Rect)
                dc.DrawRectangle(ir)

        # wx.BoxSizer, wx.StaticBoxSizer
        elif isinstance(sizer, wx.BoxSizer):
            # NOTE: we have to do some reverse-engineering here for
            # borders because the sizer and sizer item don't give us
            # enough information to know for sure where item
            # (allocated) boundaries are, just the boundaries of the
            # actual widgets. TODO: It would be nice to add something
            # to wx.SizerItem that would give us the full bounds, but
            # that will have to wait until 2.9...
            x, y = rect.GetPosition()
            if sizer.Orientation == wx.HORIZONTAL:
                y1 = y + rect.height
                for item in sizer.GetChildren():
                    ir = self.AdjustRect(tlw, win, item.Rect)
                    x = ir.x
                    if item.Flag & wx.LEFT:
                        x -= item.Border
                    dc.DrawLine(x, y, x, y1)
                    if item.IsSizer():
                        dc.DrawRectangle(ir)

            if sizer.Orientation == wx.VERTICAL:
                x1 = x + rect.width
                for item in sizer.GetChildren():
                    ir = self.AdjustRect(tlw, win, item.Rect)
                    y = ir.y
                    if item.Flag & wx.TOP:
                        y -= item.Border
                    dc.DrawLine(x, y, x1, y)
                    if item.IsSizer():
                        dc.DrawRectangle(ir)

        # wx.FlexGridSizer, wx.GridBagSizer
        elif isinstance(sizer, wx.FlexGridSizer):
            sizer.Layout()
            y = rect.y
            for rh in sizer.RowHeights[:-1]:
                y += rh
                dc.DrawLine(rect.x, y, rect.x+rect.width, y)
                y+= sizer.VGap
                dc.DrawLine(rect.x, y, rect.x+rect.width, y)
            x = rect.x
            for cw in sizer.ColWidths[:-1]:
                x += cw
                dc.DrawLine(x, rect.y, x, rect.y+rect.height)
                x+= sizer.HGap
                dc.DrawLine(x, rect.y, x, rect.y+rect.height)

        # wx.GridSizer
        elif isinstance(sizer, wx.GridSizer):
            # NOTE: More reverse engineering (see above.) This time we
            # need to determine what the sizer is using for row
            # heights and column widths.
            #rh = cw = 0
            #for item in sizer.GetChildren():
            #    rh = max(rh, item.Size.height)
            #    cw = max(cw, item.Size.width)
            cw = (rect.width - sizer.HGap*(sizer.Cols-1)) / sizer.Cols
            rh = (rect.height - sizer.VGap*(sizer.Rows-1)) / sizer.Rows
            y = rect.y
            for i in range(sizer.Rows-1):
                y += rh
                dc.DrawLine(rect.x, y, rect.x+rect.width, y)
                y+= sizer.VGap
                dc.DrawLine(rect.x, y, rect.x+rect.width, y)
            x = rect.x
            for i in range(sizer.Cols-1):
                x += cw
                dc.DrawLine(x, rect.y, x, rect.y+rect.height)
                x+= sizer.HGap
                dc.DrawLine(x, rect.y, x, rect.y+rect.height)

        # Anything else is probably a custom sizer, just highlight the items
        else:
            del dc, odc
            for item in sizer.GetChildren():
                self.HighlightSizerItem(item, sizer, 1)


    def FindHighlightPos(self, tlw, pos):
        if self.useOverlay:
            # We'll be using a ClientDC in this case so adjust the
            # position accordingly
            pos = tlw.ScreenToClient(pos)
        return pos


    def AdjustRect(self, tlw, win,  rect):
        pos = self.FindHighlightPos(tlw, win.ClientToScreen(rect.Position))
        rect.Position = pos
        return wx.Rect(pos, rect.Size)


    def DoHighlight(self, tlw, rect, colour, penWidth=2):
        if not tlw.IsFrozen():
            tlw.Freeze()

        if self.useOverlay:
            dc = wx.ClientDC(tlw)
            dco = wx.DCOverlay(self.overlay, dc)
            dco.Clear()
        else:
            dc = wx.ScreenDC()
            dco = None

        dc.SetPen(wx.Pen(colour, penWidth))
        dc.SetBrush(wx.TRANSPARENT_BRUSH)

        drawRect = wx.Rect(*rect)
        dc.DrawRectangle(drawRect)

        drawRect.Inflate(2,2)
        if not self.useOverlay:
            pos = tlw.ScreenToClient(drawRect.GetPosition())
            drawRect.SetPosition(pos)
        wx.CallLater(self.highlightTime, self.DoUnhighlight, tlw, drawRect)

        return dc, dco


    def DoUnhighlight(self, tlw, rect):
        if not tlw:
            return
        if tlw.IsFrozen():
            tlw.Thaw()
        if self.useOverlay:
            dc = wx.ClientDC(tlw)
            dco = wx.DCOverlay(self.overlay, dc)
            dco.Clear()
            del dc, dco
            self.overlay.Reset()
        else:
            tlw.RefreshRect(rect)


    def FlickerTLW(self, tlw):
        """
        Use a timer to alternate a TLW between shown and hidded state a
        few times.  Use to highlight a TLW since drawing and clearing an
        outline is trickier.
        """
        self.flickerCount = 0
        tlw.Hide()
        self.cl = wx.CallLater(300, self._Toggle, tlw)


    def _Toggle(self, tlw):
        if tlw.IsShown():
            tlw.Hide()
            self.cl.Restart()
        else:
            tlw.Show()
            self.flickerCount += 1
            if self.flickerCount < 4:
                self.cl.Restart()


#---------------------------------------------------------------------------
from wx.lib.embeddedimage import PyEmbeddedImage

Refresh = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAABehJ"
    "REFUSImdll1olNkZx3/vRzIxk5lJbMwmGHccP+JHS6VrYo3TKCvL0i0LLTRB8cbLitp6p9ib"
    "elHohVLT0BqXBnetqBQveiWF0oXiF+1FS4PUxFgbm0yYTN/JZL4nmcl7/r3IJMRlodAHDhwO"
    "z8d5/uf/PM+x+N9yADgDfAtwAAvwgafAJ8DfvsxIq3q4G86cuiHAB8C/gZLjOO/4vv8u8LWu"
    "rq4lgGQy2dTQ0JDZuXPn9snJyXmgGYgBnwMGcCzwBZb7BedbgJ+5rntk69atJdd1f/D69evX"
    "tm1bAwMDDA4ONlmWxYMHD5iYmGj0fT8BhGOx2Cezs7MdKysrfwZ+DCTXgmzMaovjOPdXs1tf"
    "nwJXgX8ODQ0plUqpXC7r9OnTAmZDodDNtra2zzba2Lb9AOj8MtjGAIVCIfX29ppDhw6Z1tZW"
    "AWpvb9fNmzf9dDqtUqmksbExPxQKCdC+ffvU29ur3t5eEw6H1wL9po7KunzgOM4/AB08eNBM"
    "TU3J8zxdunRJtm3r4sWLkqRCoaBkMilJunz5smzb1oULFzQ/P6/p6Wn19/cbQK7rvgQ+2hig"
    "Z/v27c8A9fX1yfM8JRIJJZNJzczMKJVKqVQqKZ/PK5fLqVgsKpVKaWZmRslkUolEQouLixoY"
    "GDCAotHo34H9bEijMZvNft7W1hYJBAJf9zyPeDxOR0cHoVCIxsZGarUalmVhWRbGGILBIJFI"
    "hGAwSK1WY3h4mIcPH1qVSuVue3v75cXFxQyQBzjQ09Pz3V27dn0jEon8qv5QmpmZ0crKirLZ"
    "rMrlsr4olUpF2WxW1WpVnucpGAyu4f8LYKfjOB8CBxzgSqFQ+NhxnI8zmUxfMBiMnD9/nmPH"
    "jtHY2Iht2xSLRcbHx3ny5AnPnz8nn88TCoXYtGkTxhh8f5WNExMTlMvlDtu2+4wx/cBugOeA"
    "4vG4Tp48qdHRUV+SisWicrmcJOnp06d6//331dDQINu2dfToUT169EiSlMvlVCgUJEm3bt3y"
    "BwcHdfz4cdm2rbpvXnR1dVVGRkaUy+WUz+eVTCbX95J07949NTQ0bOS6bt++LUnK5/PK5/Mq"
    "FApKp9NKpVIaHR1Vd3f3MvDCZa1nuC6+72NZFsFg8K0CkbQOA4AxBmPMWzrFYpFwOIxlWdi2"
    "jWVZAJYD/KhUKr2ztLTE48ePWVpaMocPH7Z838cYQyAQIJ/P8+rVK2ZnZ5HEkSNHGBoaIhqN"
    "sry8jG3bbN68mfv375uRkRHr2bNnjI+PO0DKAq4AvbZtNxljdnR0dMTOnDnDuXPnCIfDABQK"
    "BSYnJ5mensYYw44dO9i7dy/hcBhJVCoVRkZGGB4eJpfLzXV2ds5mMpmVarX6AqDDcZzj9cL4"
    "+f9L0+bmZgEKh8O3enp6+vbs2fN94D0HKEmqxWKxYDabPRqJRN47e/YsAwMDBINBXNfFGEOl"
    "UqFarVKtVtdhCQQCACwvL1Or1VhcXKRUKk3Ozc39cWFh4V/Ay7U32rWxVczPzyuRSMjzPHme"
    "p4WFBRUKhbcYk8lk5Hme0um0EomE0um04vG4AMVisWfAPoFl1wNsT8zNbV4jTaVSIRgMcv36"
    "daLRKFevXqWlpYVyuQxAS0sLN27cIBqNcu3aNZqamlhaWkKSABKJxBYgZoEQWEOrPenTOobq"
    "7+838Xjc7N+/X4BaWlo0Njbm5/N5ZbNZ3blzx+/s7BSg1tZWxeNxxePx9fYO3AUaV69brwOg"
    "qz4s1guqtbX1t+Fw+NfA7IkTJ5TL5ZTJZHTq1CkBb4BfAp9ttHFd93dA95pvF+AgNPwVksaY"
    "HwIV13W/2d3dnX/z5s1Pd+/e7TQ3N+9LJpPdd+/exXVdPM/Dtu2XxpiRWCzWJOmrc3NzbbVa"
    "7S8rKyuXgASrqBh+AnY9i43z+aM6bbf29PR8LxAI/AlQd3f38rZt25YdxxHwB8dxvg28C+wF"
    "vrMOS30MrGdwBSytDmgLMBb8fo1eU1NT7cAE8JVEIrHx2zLt+/5/gJm66mT9oharPwsL4L/1"
    "GXlKb/xX4wAAAABJRU5ErkJggg==")

Find = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAABgRJ"
    "REFUSIm1lU1oG9sVx/9z5440+kBWJUvGDZESXuskZPMIwVaoybNp4niXEChdJPDAIWk+IGnA"
    "i1Ioz9208apk10WcZFMI3Zgugrww6cNxKcakdoK/ghU7k5EsW2PLljXfc2duFw/5uaRv1x4Y"
    "uHc4c3/38P+fM8D/OYTDm7Gxsd/4vv/H169fQ5IkAIDjODh16hSy2ey3t27d6geAJ0+eFDVN"
    "G1xYWEA4HAYAeJ6H3t5eUEp/f+PGjZHPSOPj48P37t1j+XyeAzh4QqEQLxQK/Pr1639v5V67"
    "dq3Y29t7kEMI4aIo8lwux2/fvs3Gx8d/28qlrYXv+18RQsTNzU129epVWigUUC6X8fz5c8zN"
    "zUEQBKuVu7a2Zs7MzOD06dO4c+cOJicnUavVMDs7ywRBoIyxfgB/+A8ApXS7Xq8jkUjQCxcu"
    "4MqVK1hbW8OrV6/w6dMndHV1fXHmzJmvCSGs2WyeePPmDU6ePImbN2+CUgpVVVEqleju7i4o"
    "pdufVSDLMhhj0DQNMzMz2Nragu/72N7ehizLLJ1Od3me91wQBKRSKSSTSW9+fl56/PgxFhcX"
    "IQgCNE2DbdsIhUL4DOC6LjjnIIRAFEXU63VYloUgCBAEAVUUJTBN0wGAWCwW5pxLtm1jdXUV"
    "mqYhnU4fGIMxdgAgrcWHDx+aiqJAFEVks1l4nodisQjHcdDT04NsNvuPYrEYLRaL0Ww2++rc"
    "uXMwDAMTExM4duwYGGNwXRfVahUrKysHABEA7t69+7u3b9/ekmU50t3dDV3XMTExgUqlAlmW"
    "cfbsWdi2Td+9e3cEwIWurq6vkslk29LSEmq1GjRNQz6fR0dHByRJgqqq06VS6eUBoLu7+2+r"
    "q6s/2traYslkkszOzkJRFMiyjFwux8LhMNF1PWGa5rl4PP6zeDze5rouDMNg9Xqd7O3tQRRF"
    "ZDIZqKqKcrncdv/+/a2pqalFCgDhcPhjpVL50jRNWigU0N/fj0uXLkFVVayvr9OFhYVSNBot"
    "p1KpPgAol8tTjUajI5/PnxgYGIAoitB1HdVqFe/fv/dyudxPG43GXwD8FQDw8OHDuVQqxQcG"
    "BnitVuOGYfD19XU+PDzM29raOIBhAJFDDZgEcLuvr48risKbzSbXNI2PjIxwWZZ5LpfjDx48"
    "WD5wESEElFLoug5VVRGJRFAqlaDressZDIB7qPE9AL7jOFBVFYZhYGNjA3t7e5AkCYIggBDy"
    "vU0dx3FM04Smadjc3IQsy1heXoZpmq1Z8ysAg4cA4wB+7DgOKpUKPM/DysoKdnZ2YJomJEmC"
    "4zguAIhjY2MjL168+DmAeKFQQGdnJ2zbRrVaRb1ex/Lyssc57+jp6fnJ8ePHkc/ncfTo0S/K"
    "5XI2kUh43d3douu6KJfLkCQJ7e3taDQaqFQq4qNHj2KUMfbN4uIiLl686J8/f16sVqtIJpNw"
    "HAecc9i2LeVyOQwODm7ruv4tgCAej/dVq9WsYRiSaZpwHAe6riMajeLy5ctgjPkvX75sZ4x9"
    "Q6anp8E5RyaTET3Pw87ODmzbxs7ODhhjoJSCEIL9/f1/jY6O/mJ0dPSXzWbzn5RSuK6Ler0O"
    "27bRaDSgKAosy0ImkxEBYHp6GqQlimVZYIyBEALHcUAIgSB897vgnINzHjqkQbg1VgRBgOM4"
    "EAQBoVAInufBsr4bvJIkodUHKJVKkGUZrutid3cXhmHA9338UFBKYRgGVldXEQqFYJomLMvC"
    "3NwcSqXSwVyirRuKoohUKoVYLIZMJoOPHz8iCALIsvxfQUEQgFKKzs5OdHR0QFVVbG9vgzEG"
    "y7K+t2kkEkEQBFBVNVhaWiLRaBTxeByKoiAIAtRqNT+dTosA2g8d3k4pRb1e9+fn58V0Og1V"
    "VdFsNsE5h6Ioge/7JBKJgFqWNX/kyJETGxsbkampKXDOEQTBQYmSJInxeBwAploAQsjrWCx2"
    "VpIkcXJyEr7vQ5Kkw7qRzs5Ox7KsZQEAhoaG5iKRyJctcQ5HIpFAo9H487Nnz+4cfj80NPSn"
    "RCLx6/39/c++kWUZtm2vPH369NQPivi/in8Df18XwyUA5+QAAAAASUVORK5CYII=")

ShowSizers = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAABChJ"
    "REFUSIm1lU1oXFUUx3/33nn3zbx5mcm0M5OZxGlNAn6XUqlRKIIWKrXqRlCLOwvitrooqKDg"
    "RhcirkSpigSECl0WBa104UKoSIoLS8TWlja1NTOTyUzz5s37uNdFWpumJSktHriry//8zj38"
    "z7nwP4dY5/7BUqn0quu62621i9cJhSjGcTzTarUOAr/dFn1sbGy/N+SdksgoQ2bh2mFBQpTz"
    "vdP1ev3AWjkya10qpe4yPbPZ3GeUecEMswQoYK4I3w+wzWiz3qgbtw0AQoHoswWfNxyYLYE2"
    "8GcVfr8IzU5gjOnfCWA5YuCvHPw0CRkLhGDjW5LeGiAFWjGcaYKyUHLAwvoeWQcgpczZjM3z"
    "A3CiD5fPAlikFXRicNy8lNJbK4das/A0VdKVJd/4oWqJZhSGVcJUeH3n5JA/NGe1OhEG/W+i"
    "KJq9LUAURe1KuVJQrrqnn4YVrXXkOE5gFCpf8NteLvdts9k8AgRXJDf0bC1ArlgsTiVJsqfX"
    "6+1K03RQLpenfd//tdvtbh0MBvdaaxe01pcGg8FFlq1wU8jNoqiUeq5Wqx3SWlutdWvTpk3v"
    "ANuBbY1G423Xdecdx7EjIyOHlVLPA6X1kl4lK6XU7rGxsU+UUkue54XlcvlL4LEVL36kUql8"
    "ns/nl6SUwejo6EGl1DNcM81/r7ihRfl8fmehUHil2Wzu1Vr3Xdc9lCTJZ4PB4DhXzAlcAOa0"
    "1koIMdntdqdKpZInhGjHcXx6ZT4FjDQajR21Wm2L7/sPJ0nyYr/ff1ZKOV8ul2eTJDnS6XSO"
    "sjwN4mp1cRw3s9lssVQq1cIwVEmSbNVa+9VqNTsyMjLh+/744uJiT3ied8BxnJcx6QMS0wkH"
    "UaEfJe6w4mc8v5AINWOS+KMgCGZWVuZ53taMlPtRaqovTRAv9LaTdQIn5y0oYzZkkaejOJ6m"
    "Xq8fk0IkCGVxci2UnsfNz1MatSDtUN7rT0xMvLa6lePj4/v8wlAfsCVUuJFMZxh1eQMqyIFF"
    "irRerx/LGGMCYa3ioV1f8el30/w4k8V178bNfMjHL3mcPREA0U1MES3ZNK2R6Z9katpgkpBU"
    "jFLovcnJRz8QF54wxgTXVoUQl9iBzz/bniQlT39JIdecQywICfEwQwFkd4KqdAmDS8gKEMLK"
    "XZTaOVImsbyOpM1QXiPkmgBAZBApeOEsS5OSjD8gJYsG6AFkhBA5C3D6+G72vudTGYXg8gYQ"
    "0J6DDB4sK67LLISjhXQ6xLm3+OXxU6RuBGxEM0sPLFkhRC5jjDmntD5D2N4jDr8LsMCV+bCQ"
    "t8Xhs0mStFYD4jhu55B/LEpx//vi/A5gieWdJLBoV+m2MeacAJ6uVqt7pZQNa+11v5MQohAE"
    "wdFut/sFcH4VY9T3/X2+7z9lre2t0mXTNP17fn7+6/V6fMfxL1klnkaQRVDaAAAAAElFTkSu"
    "QmCC")

ShowFilling = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAAAZ5J"
    "REFUSIntlc9KAkEcxz+tu7OxLRSuGUXtIfUpukW9QJcO3nqELoLXLr2Bt47dRBAqgxJ6BqMw"
    "PJe0FEliaLNrFxV1zXatY9/TMDPf72eG3/yBf/2guaH2DrALrADtGfME8AKUgCsAdWhwX1XV"
    "bSnlMtCZEaApivrqeXJxEmBDSrl+dHQoNjf3OD9/xjSDJ5vmMre3Z1xeHi8AG/3+YcAH8GEY"
    "SbG6uoVtP2IYwQFLS2s8PT0MciYBALi5uUfTrrEsB88LDtB1C027A+h+N6cAvBUK+e6surg4"
    "6wLvvSwAlHFKu/0ZfNkBvD6AEGJmwCSvrwaNRgOAZrMZKtw0zYF3KiCXy1Eul2m1WqEAhmFQ"
    "q9VgrMg+QKVSoVqt4oU5QoCiKHQ6/vvpA2QyGdLpNPV6PRQgHo+Tz+fJZrPTAYlEgmQyiW3b"
    "oQBCCFKpFIy+b36ArusDQ1j1vCM18B3TaDQaOrgvy7Jgyg7mgflisQiA4zihwmOxGKVSCUDv"
    "ZTFOO/mL5zoSiby6rnsFHMDoDk6llA6//HBc1+1/OP8Kpi8497f1tG0HzQAAAABJRU5ErkJg"
    "gg==")

Icon = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAAALhJ"
    "REFUWIXtl80SgyAMhHeR99Y+eJseLAdHbJM0TjiwN2dI+MgfQpYFmSqpu0+AEQDqrwXyeorH"
    "McvCEIBdm3F7/fr0FKgBRFaIrHkAdykdQFmEGm2HL233BAIAYmxYEqjePo9SBYBvBKppclDz"
    "prMcqAhbAtknJx+3AKRHgGhnv4iApQY+jtSWpOY27BnifNt5uyk9BekAoZNwl21yDBSBi/63"
    "yOMiLAXaf8AuwP9n94vzaTYBsgHeht4lXXmb7yQAAAAASUVORK5CYII=")

CollapseTree = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAAAf9J"
    "REFUSIm9lD9r20AYhx9Fxe5SKDIynVJodFMaChkC6eBCJ+WWdulSb1o6RR8gH6AfQF5aSksX"
    "aQ0eajwVkinO1MFk0YXg0uLSISZDoQhcddAf5MR241buD4670/tyD+/vXh0sWdqUb1vAQ2Bj"
    "Suwb0E7Xx38DaAKPgTuAnJLfSSEAn4DWIoAm8ByQruti2zYAlmUBoJSi2+3ieV4R1v0TJANs"
    "AS8Ap9PpYFkWUSSuJFcqIUopAKSUAO+A18yxS0/nZ8AD13WFbdtEkWB9Hep1ODqC0QgMA8bj"
    "GqYJhmGgaRq9Xm8I3AY+zgKspPMGIIuHZ6qn4/Q02UeRIIpEZqEkua+ZWpkXLEM3ipvEe9C0"
    "ad2bqN+P89zr6P9WoJRidVXQ78e55/U09h1YW5vMTTWcB8i66B7wq1arie1ti/G4hmEknfNl"
    "BD8Kh1cqIbp+ThAEVKtVBoNBCziZBfjX/wDgEHg0C7D0O8gs+grcAm76vi80TcM04eJCYZqg"
    "6+ecnR0TBAGu6+L7PlJKhAgJQ+6SvF/vrwNsAm+BD0B8eTQajXztOMT7HnFrL48fpGNCizzX"
    "Q5IXdDfdN/Y92Hna5s2rJ+y+zPPm3skiOoCkip+f23Fr70o1pWgCkoGKkKV3URnKqyjaxTKs"
    "omCX4+SQ0pS1aew4xJub5VZwGZQdfv83yOfTR/iA1xwAAAAASUVORK5CYII=")

ExpandTree = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAAAepJ"
    "REFUSIm1lDFr20AYhp+ri+iSxUUhW4dIU9wlQ4YOKnRSj4K7Z9NSCFg/ID+gP0BZWkpLF3kN"
    "Giq0pkOpMxWSTFIIFIqHQCFbUQnXQTpFslXXptYLx90nvdzD9913Bx1LtHzbA54Aj1v+nQFf"
    "yvXpMoD7M/E+8AzYAmSLP66BbSBcBTACXED6vo/rugBYlgVAlmUkSSKDIND+rXJeCNEl2gNe"
    "AV4cx1iWRZ7bc2bDSMmyDAApJcAH4C0LytUr5wPA8n3fdl2XPLfZ2YHNTbi+vjPf3j7ENKHf"
    "7yOEYDKZTIHfwNe/Ae7V0pX1zbUGA8FgILi8LOI8t8lzW5dQ0t4Mc4DO1ADoA724ACEEQtx1"
    "8XBYZDLrXQnQhRoA3SEAUaSIItWIz89Vm3e6DOAMiJMkwTBSALa3i6Gl14aRYhgpSZLgOA7A"
    "t0WA/70HAJ+Bp//KoDPpi/YD2AAehGFoCyEwTbi5yTBN6PV+cnV1yng8xvd9wjBESoltp6Qp"
    "jyjer4/LAPeB98AnQM0Ox3GqteehjgPU0WH1/6QcDa3yXE8pDnRUxs5xAM9fRrx7M2T0uvIt"
    "PJNVdAJFFr++R+rocC6btagB0aA6pPMuWoeqLOrlootSUSuX51WQtUm3qfI81O7uejOYBenN"
    "X/wBVz/ONKbGYPkAAAAASUVORK5CYII=")

HighlightItem = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAAAQZJ"
    "REFUSInFlTGSgjAUhv8XuIRl9ga0XgmuoI5abwfH8Ai22GlJZ0otuQD5t3DZ1V1CwgzgP8OQ"
    "QCZfkv+9FxEVYUrFbYO2oW+wqEiGAtRzhyRIQh9eH+RXAMBmvfIuohcwheLnTnZ6vM3NjAaQ"
    "1mTahvrAHwCzj+BJVI83sesHAMjRM3OVgNkFm/WK292+EzKvB86zr5Lu76b2AubdAbqMda0+"
    "UOIqFdY2lKMHYGrw06DL3Tbrxzmi/Iq0JNLyO/Pxm/Uze/BXVRIUKajvKM6AXuh/kfjeHTC7"
    "TAdw1RfahmlJFOewgtjvQY/0QgeNe3MUOVQsw2/OwQBRkQy5Op2lYixN7sEXVhRd4PXVHvwA"
    "AAAASUVORK5CYII=")

EvtWatcher = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAABHNCSVQICAgIfAhkiAAABwxJ"
    "REFUSIltlltsXFcVhr+zzzlzztw949vMxGPHl3Hiqk6I07RJhcBGoUK0VSuaSFwahAChSkiV"
    "QCotlQAJP/FAeKFPSChRJUAtqZACjUpJBCqUlEY0l7HjpraT+DKejDPjmfHcznXz0Bg1iPWy"
    "ttbD/629tbT/pXB/KIAEVCADxIBdwMi93AX4QBlYA5aBIlADNvg/oXzirAIeoAcCgVFN0460"
    "Wq0DuVzuc7VaLVspl52xsTHN9TwWFxfdbDarKYqysLKy8tdYLHalXq9fBG7fa0Dcy/8F7BSi"
    "hmEcdF33Cc/znt63b1//1NRUMBQKqeFolOHRMTzbYmlxEcdx2Nrast+7eNFaWl6+bZrm667r"
    "nnNd9ypg7Whq9yA+EInH4w/VarUTmqY9MTMz03vs2HFS/X1eOjvojmUHFM22sIMRXN/nxlxe"
    "lutVdXLqYPTM73774KVLl+KKosSCwaBst9vXdiDqzg0MwzjSbDafBR6bmppKv/TSS+7MzLTs"
    "7elTUwMp0ed7wlwvCnOwX8QjQaHHYiKdSCqHcjkvkckwl893VSqVbtd1DV3XC77vFwFFvff2"
    "4wjxFen7xx46dCjzwosvuuO5nNbT0yuSXWFk0KSx3sa53aLal6QtBVFF0GdZinBsoaZSjO7e"
    "LfP5fG+1Wu0TQmwbhnHLdd1tFRgIhULTtmV9Z3p6Ovvyyy+7ubExbWhoCEWoBBWPSzWdM4UE"
    "C+U011YM3gvojCwskZAOzclxkgFDCYZCyv79+735+fnuUqkUjUQii51Op6YCWcdxnsiNjR0+"
    "fuKE+cUjR+Sg7wurt5eA6bB2I8b3/1Hn3EKFY+UYzS2H2Y1bbFlpDuaydJsbBAo1woMDaLqO"
    "lFIpFApsbGysxGKxD1XgwJ7x8ecOPPLI4NMPP+w9ODqq2uEwkSvLrJWzfP5PS/SoJb7xpSy/"
    "GNUJd6noWoK3bn1E+a7D4XiaaJ+FpwWpVatKu9Vyt5vNrmaj4ZZKpUsCGCkWi/1d8TiZ3cMS"
    "VcHvWLyvJHj2gy0mBz1+/7VJHvRXmDv5dc6e/jF1c4HUeJjXS02ev9HhcjCDY2ok0ymGRkak"
    "qigU79xJCyFGNGBXrV4XXaEQxsiI4mnglRv8OZbjxvYax2cGaEfavPnTU/CrV1kH1qv/5Oi3"
    "XyG1Z4wr60Xenw+xJ75JVzyAOzSkqIEAtmUZpmkOCqBrYmJCGx4bI9SxICCoZrJEFySvtNL8"
    "xoqQ+uGP+OXJn0EkigiY8Ie/cPvfeY4GVX7QTMFNG+PGBqwX0KWv7H3gAbIDA4FOp5PQAN+2"
    "LGlLiV6vozp1lOgAvq6wrnkc3FQpVvtYBYRl47s2AJoQtARsuS6RqE7j0Ql0Ddy7VdxOB8u2"
    "JeALoLy0vOzevL5AvbsLPJPEu9exd8NPomtcr3jseupJeOYxcCyQEp47wa7Dh3hrq8PJ3iLa"
    "uABVQVU1LMeR1+fnKZVKlmmamxqwtiuT8betDvXFRUlmEMZ7eDzZ4vrGCK9duUx/LsL0t37N"
    "+sQ7aIZB6tHPkF8pYJeafDfdw5Pr8/j9uymUNvlwfl5quk4ymbQrlcqKBiwJIZaqtVpvcXVd"
    "jA4OSW9olzJuW/xcKxJWDV5d1Ulne/jqp79Mx4U3i228lQDP70nyvf0msZaPVymzcrcsC8Wi"
    "aGxvI4T4CFhSgWC9Xk90ms29vcPD4eHJSS9gu0JtO8Q6qxwaHmYpFmF5ZZPHb0VQq5Lzosg3"
    "H0jzwsEg+qhLM9GLslGg4jj+uQvn1fNvv32rVCqdicfjH6iAHg6HzVKpNNWoVvt6+/tlJBAQ"
    "aduiMZ4jNqiwt0cwkOjBNyVG3OXop7p5Zp9NT9ii09bwOy3my2X+/u678tXTp8Xq2trlZDL5"
    "RrVaXVWBhuM4DdM01ZXV1ZFrV68md4+O+mp/v5Lo6kJKlRgukynJ/u46hwvzjOdMjFAAPAXL"
    "arO4tMTfLlxgdnZWFIvFq4ZhnG40GheAzZ3vuuq6bktVVSqVSnd+bq53ct8+T1NVGYnElGg4"
    "pPjSQ8VD85r43XGMaIzKZlmura95Fy9eFLOzs9RqtX8JId5wXfeP8PFk7wAUoBQMBu86jhOs"
    "1Wp983Nz3YBotZredqPhSV/BcjzZDGjybrPl5/Nz3rWrV+XZs2e1U6dO+Xfu3MkLId7QNO01"
    "z/MWd/z9fy3TCIfDeyzL+oLruk8NDw8PT05O9qiqqgcCAbl3YkJxfJ/r165JVQil0Wy2L1++"
    "XC4WiwuGYZyRUp63bfsm4O5oftL0dyAiGAxmDMM4UK1WJ4eGhj67vb09tFWtkk6lDN9xZHFz"
    "08pkMhJYKBQK7ySTyXylUrkClO51vmPD920V90GA/lgsFqrX6/26ro8oijJo23YC8E3TvGvb"
    "9m0hxHI4HC7XarXmJ8Th49UHgP8A40NGDcCfTKIAAAAASUVORK5CYII=")
